""" Citrix Netscaler Nitro API accessor """

import urllib
import urllib2
from nsutil import *
from nsexceptions import *

__version__ = "0.0.2"


class NSNitro:

    """ Main class """

    __ip = "1.2.3.4"
    __user = "api_user"
    __password = "api_user"
    __baseurl = "http://1.2.3.4/nitro/v1/config/"
    __sessionid = ""
    __loggedin = False
    __initialized = False
    __contenttype = "application/x-www-form-urlencoded"
    __postheaders = {
        'Cookie': 'sessionid=' + __sessionid,
        'Content-type': __contenttype}

    def __init__(self, ip, user, password, useSSL=False):
        """ Constructor: ip - LB ip, user - LB username, pass - LB password """
        self.__ip = ip
        self.__user = user
        self.__password = password
        self.__baseurl = "%s://%s/nitro/v1/config/" % (
            'https' if useSSL else 'http', ip)
        self.__initialized = True

    def get_url(self):
        """ Returns base url for nitro API. Mostly useful for debugging """
        if not self.__initialized:
            raise NSNitroError("Not initialized.")
        return self.__baseurl

    def get_sessionid(self):
        """ Returns sessionID that LB gave us after logging in """
        if not self.__initialized or not self.__loggedin:
            raise NSNitroError("Not initialized or not logged in.")

        return self.__sessionid

    def login(self):
        """ Logins to the LB using the credentials give to constructor """
        if not self.__initialized:
            raise NSNitroError("Not initialized.")

        payload = {"object": json.dumps(
            {"login": {"username": self.__user, "password": self.__password}})}
        try:
            nsresponse = self.post(payload)
            self.__sessionid = nsresponse.get_response_field('sessionid')
            self.__postheaders = {
                'Cookie': 'sessionid=' + self.__sessionid,
                'Content-type': self.__contenttype,
                'Set-Cookie': 'NITRO_AUTH_TOKEN={0}'.format(self.__sessionid)}
            self.__loggedin = True
            return nsresponse.get_json_response()
        except SyntaxError:
            raise NSNitroError("Could not parse LB response.")
        except urllib2.URLError, ue:
            raise NSNitroError("Error logging in!" + str(ue))

    def post(self, payload):
        try:
            payload_encoded = urllib.urlencode(payload)
            req = urllib2.Request(
                self.__baseurl, payload_encoded, self.__postheaders)
            response = urllib2.urlopen(req)
        except urllib2.HTTPError, e:
            try:
                NSNitroResponse(e.read())
            except AttributeError:
                raise NSNitroError(
                    "Could not send post request: %s, %s" % (e.code, e.reason))
        return NSNitroResponse(response.read())

    def put(self, payload):
        try:
            opener = urllib2.build_opener(urllib2.HTTPHandler)
            request = urllib2.Request(self.__baseurl, json.dumps(payload))
            request.add_header('Cookie', 'sessionid=' + self.__sessionid)
            request.get_method = lambda: 'PUT'
            response = opener.open(request)
        except urllib2.HTTPError, e:
            try:
                NSNitroResponse(e.read())
            except AttributeError:
                raise NSNitroError(
                    "Could not send put request: %s, %s" % (e.code, e.reason))
        return NSNitroResponse(response.read())

    def get(self, url):
        try:
            opener = urllib2.build_opener()
            opener.addheaders.append(
                ('Cookie', 'sessionid=' + self.__sessionid))
            response = opener.open(url)
        except urllib2.HTTPError, e:
            try:
                NSNitroResponse(e.read())
            except AttributeError:
                raise NSNitroError(
                    "Could not get resource: %s, %s" % (e.code, e.reason))
        return NSNitroResponse(response.read())

    def delete(self, url):
        try:
            req = urllib2.Request(url)
            req.add_header('Cookie', 'sessionid=' + self.__sessionid)
            req.get_method = lambda: 'DELETE'
            response = urllib2.urlopen(req)
        except urllib2.HTTPError, e:
            try:
                NSNitroResponse(e.read())
            except AttributeError:
                raise NSNitroError(
                    "Could not send delete request: %s, %s" % (e.code, e.reason))

        return NSNitroResponse(response.read())

    def logout(self):
        payload = {
            "object": json.dumps(
                {"logout": {}, }
            )
        }
        try:
            nsresponse = self.post(payload)

            del self.__sessionid
            return nsresponse.get_json_response()

        except NSNitroError, nsresponse:
            raise NSNitroError(nsresponse.message)



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSVLANInterfaceBinding(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSVLANInterfaceBinding, self).__init__()

        self.options = {
            'id': '',
            'ifnum': '',
            'tagged': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSVLANInterfaceBinding.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "vlan_interface_binding"

    def set_id(self, id):
        self.options['id'] = id

    def get_id(self):
        return self.options['id']

    def set_ifnum(self, ifnum):
        self.options['ifnum'] = ifnum

    def get_ifnum(self):
        return self.options['ifnum']

    def set_tagged(self, tagged):
        self.options['tagged'] = tagged

    def get_tagged(self):
        return self.options['tagged']

    @staticmethod
    def add(nitro, resource):
        __resource = NSVLANInterfaceBinding()
        __resource.set_id(resource.get_id())
        __resource.set_ifnum(resource.get_ifnum())
        __resource.set_tagged(resource.get_tagged())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        __resource = NSVLANInterfaceBinding()
        __resource.set_id(resource.get_id())
        __resource.set_ifnum(resource.get_ifnum())
        __resource.set_tagged(resource.get_tagged())
        nsresponse = __resource.delete_resource(
            nitro, object_name=__resource.get_id())
        return nsresponse

    @staticmethod
    def get(nitro, resource):
        """
        Use this API to fetch service resource of given name.
        """
        __resource = NSVLANInterfaceBinding()
        __resource.set_id(resource.get_id())
        __resource.get_resource(nitro, object_name=__resource.get_id())
        return __resource



from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'


class NSIPTunnel(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSIPTunnel, self).__init__()

        self.options = {
            "remote": "",
            "remotesubnetmask": "",
            "name": "",
            "sysname": "",
            "local": "",
            "protocol": "",
            "type": "",
            "encapip": "",
            "channel": "",
            "secure": ""
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSIPTunnel.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "iptunnel"

    # Read/write properties
    def set_name(self, name):
        '''
        The name of the ip tunnel.
        Minimum length = 1
        '''
        self.options['name'] = name

    def get_name(self):
        '''
        The name of the ip tunnel.
        Minimum length = 1
        '''
        return self.options['name']

    def set_remote(self, remote):
        '''
        The remote-ip or subnet of the tunnel.
        Minimum length = 1
        '''
        self.options['remote'] = remote

    def get_remote(self):
        '''
        The remote-ip or subnet of the tunnel.
        Minimum length = 1
        '''
        return self.options['remote']

    def set_remotesubnetmask(self, remotesubnetmask):
        '''
        The remote-subnet mask of the tunnel.
        '''
        self.options['remotesubnetmask'] = remotesubnetmask

    def get_remotesubnetmask(self):
        '''
        The remote-subnet mask of the tunnel.
        '''
        return self.options['remotesubnetmask']

    def set_local(self, local):
        '''
        The local-ip of the tunnel.
        '''
        self.options['local'] = local

    def get_local(self):
        '''
        The local-ip of the tunnel.
        '''
        return self.options['local']

    def set_protocol(self, protocol):
        '''
        The IP tunneling protocol.
        Default value: IPIP
        '''
        self.options['protocol'] = protocol

    def get_protocol(self):
        '''
        The IP tunneling protocol.
        Default value: IPIP
        '''
        return self.options['protocol']

    def set_secure(self, secure):
        '''
        Secure communication using IPSec.
        Default value: NO
        '''
        self.options['secure'] = secure

    def get_secure(self):
        '''
        Secure communication using IPSec.
        Default value: NO
        '''
        return self.options['secure']

    # Read only properties
    def get_sysname(self):
        '''
        The name of the ip tunnel.
        '''
        return self.options['sysname']

    def get_type(self):
        '''
        The type of this tunnel.
        '''
        return self.options['type']

    def get_encapip(self):
        '''
        The effective local-ip of the tunnel. Used as the source of the encapsulated packets.
        '''
        return self.options['encapip']

    def get_channel(self):
        '''
        The tunnel which is bound to a netbridge.
        '''
        return self.options['channel']

    @staticmethod
    def add(nitro, iptunnel):
        __iptunnel = NSIPTunnel()
        __iptunnel.set_name(iptunnel.get_name())
        __iptunnel.set_remote(iptunnel.get_remote())
        __iptunnel.set_remotesubnetmask(iptunnel.get_remotesubnetmask())
        __iptunnel.set_local(iptunnel.get_local())
        __iptunnel.set_protocol(iptunnel.get_protocol())
        __iptunnel.set_secure(iptunnel.get_secure())
        return __iptunnel.add_resource(nitro)

    @staticmethod
    def delete(nitro, iptunnel):
        __iptunnel = NSIPTunnel()
        __iptunnel.set_name(iptunnel.get_name())
        nsresponse = __iptunnel.delete_resource(nitro, object_name=__iptunnel.get_name())
        return nsresponse

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all iptunnel resources that are configured on netscaler.
        """
        __url = nitro.get_url() + NSIPTunnel.get_resourcetype()
        __json_iptunnels = nitro.get(__url).get_response_field(NSIPTunnel.get_resourcetype())
        __iptunnels = []
        for json_iptunnel in __json_iptunnels:
            __iptunnels.append(NSIPTunnel(json_iptunnel))
        return __iptunnels



from nsbaseresource import NSBaseResource

__author__ = 'zojoncj'


class NSSystemCMDPolicy(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSSystemCMDPolicy, self).__init__()

        self.options = {
            'policyname': '',
            'action': '',
            'cmdspec': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSSystemCMDPolicy.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "systemcmdpolicy"

    def set_policyname(self, policyname):
        self.options['policyname'] = policyname

    def get_policyname(self):
        return self.options['policyname']

    def set_action(self, action):
        self.options['action'] = action

    def get_action(self):
        return self.options['action']

    def set_cmdspec(self, cmdspec):
        self.options['cmdspec'] = cmdspec

    def get_cmdspec(self):
        return self.options['cmdspec']

    @staticmethod
    def add(nitro, systemcmdpolicy):
        __systemcmdpolicy = NSSystemCMDPolicy()
        __systemcmdpolicy.set_policyname(systemcmdpolicy.get_policyname())
        __systemcmdpolicy.set_action(systemcmdpolicy.get_action())
        __systemcmdpolicy.set_cmdspec(systemcmdpolicy.get_cmdspec())
        return __systemcmdpolicy.add_resource(nitro)

    @staticmethod
    def update(nitro, systemcmdpolicy):
        __systemcmdpolicy = NSSystemCMDPolicy()
        __systemcmdpolicy.set_policyname(systemcmdpolicy.get_policyname())
        __systemcmdpolicy.set_action(systemcmdpolicy.get_action())
        __systemcmdpolicy.set_cmdspec(systemcmdpolicy.get_cmdspec())
        return __systemcmdpolicy.update_resource(nitro)

    @staticmethod
    def delete(nitro, systemcmdpolicy):
        __systemcmdpolicy = NSSystemCMDPolicy()
        __systemcmdpolicy.set_policyname(systemcmdpolicy.get_policyname())
        nsresponse = __systemcmdpolicy.delete_resource(nitro, object_name=__systemcmdpolicy.get_policyname())
        return nsresponse

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all cmdpolicy resources that are configured on netscaler.
        """
        __url = nitro.get_url() + NSSystemCMDPolicy.get_resourcetype()
        __json_systemcmdpolicies = nitro.get(__url).get_response_field(NSSystemCMDPolicy.get_resourcetype())
        __systemcmdpolicies = []
        for json_systemcmdpolicy in __json_systemcmdpolicies:
            __systemcmdpolicies.append(NSSystemCMDPolicy(json_systemcmdpolicy))
        return __systemcmdpolicies



import json
from nsexceptions import *


class NSNitroResponse:

    """
    Generic class for accessing LB response dictionary.
    Can provide string response back and a parsed dictionary
    """
    __jresponse = False
    __sresponse = False
    errorcode = -1
    message = False
    failed = False

    def __init__(self, response):
        """ Constructor. reponse - string response """
        self.__sresponse = response
        self.__jresponse = json.loads(response)
        self.__parse_response()

    def get_json_response(self):
        """ Returns LB response as parsed dictionary """
        return self.__jresponse

    def get_string_response(self):
        """ Returns LB response as a string """
        return self.__sresponse

    def __parse_response(self):
        self.errorcode = self.__jresponse['errorcode']
        self.message = self.__jresponse['message']
        if self.errorcode != 0:
            self._raise_exception()
            self.failed = True

    def get_response_field(self, field_name):
        """ Returns field_name of parsed JSON dictionary """
        if field_name in self.__jresponse:
            return self.__jresponse[field_name]
        else:
            return []

    def _raise_exception(self):
        """
            Lookup specific exception based on errorcode in response,
            if that fails raises NSNitroError.
        """
        exception_class = NSNitroExceptionClassMap[self.errorcode]
        raise exception_class(self.message, self.errorcode)



from nsbaseresource import NSBaseResource


class NSAcls(NSBaseResource):

    def __init__(self):

        super(NSAcls, self).__init__()

        self.options = {}
        self.resourcetype = NSAcls.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "nsacls"

    @staticmethod
    def renumber(nitro):
        """
        Use this API to renumber nsacls.
        """
        __nsacls = NSAcls()
        return __nsacls.perform_operation(nitro, "renumber")

    @staticmethod
    def clear(nitro):
        """
        Use this API to clear nsacls.
        """
        __nsacls = NSAcls()
        return __nsacls.perform_operation(nitro, "clear")

    @staticmethod
    def apply(nitro):
        """
        Use this API to apply nsacls.
        """
        __nsacls = NSAcls()
        return __nsacls.perform_operation(nitro, "apply")



from nsbaseresource import NSBaseResource
__author__ = 'robison'


class NSLBMonitor(NSBaseResource):

    # Managing monitors for lb vservers
    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSLBMonitor, self).__init__()
        self.options = {'monitorname': '',
                        'alertretries': '',
                        'destip': '',
                        'destport': '',
                        'deviation': '',
                        'dispatcherip': '',
                        'dispatcherport': '',
                        'downtime': '',
                        'dup_state': '',
                        'dynamicinterval': '',
                        'dynamicresponsetimeout': '',
                        'failureretries': '',
                        'firmwarerevision': '',
                        'httprequest': '',
                        'inbandsecurityid': '',
                        'interval': '',
                        'iptunnel': '',
                        'lrtm': '',
                        'lrtmconf': '',
                        'maxforwards': '',
                        'multimetrictable': '',
                        'query': '',
                        'querytype': '',
                        'radaccounttype': '',
                        'radframedip': '',
                        'radnasip': '',
                        'recv': '',
                        'respcode': '',
                        'resptimeout': '',
                        'resptimeoutthresh': '',
                        'retries': '',
                        'reverse': '',
                        'rtsprequest': '',
                        'secure': '',
                        'send': '',
                        'sipmethod': '',
                        'snmpversion': '',
                        'state': '',
                        'storedb': '',
                        'storefrontacctservice': '',
                        'successretries': '',
                        'tos': '',
                        'transparent': '',
                        'type': '',
                        'units1': '',
                        'units2': '',
                        'units3': '',
                        'units4': '',
                        'validatecred': '',
                        'vendorid': '',
                        }

        self.resourcetype = NSLBMonitor.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "lbmonitor"

    def set_monitorname(self, monitorname):
        self.options['monitorname'] = monitorname

    def get_monitorname(self):
        return self.options['monitorname']

    def set_alertretries(self, alertretries):
        self.options['alertretries'] = alertretries

    def get_alertretries(self):
        return self.options['alertretries']

    def set_destport(self, destport):
        self.options['destport'] = destport

    def get_destport(self):
        return self.options['destport']

    def set_destip(self, destip):
        self.options['destip'] = destip

    def get_destip(self):
        return self.options['destip']

    def set_deviation(self, deviation):
        self.options['deviation'] = deviation

    def get_deviation(self):
        return self.options['deviation']

    def set_dispatcherip(self, dispatcherip):
        self.options['dispatcherip'] = dispatcherip

    def get_dispatcherip(self):
        return self.options['dispatcherip']

    def set_dispatcherport(self, dispatcherport):
        self.options['dispatcherport'] = dispatcherport

    def get_dispatcherport(self):
        return self.options['dispatcherport']

    def set_downtime(self, downtime):
        self.options['downtime'] = downtime

    def get_downtime(self):
        return self.options['downtime']

    def set_dup_state(self, dup_state):
        self.options['dup_state'] = dup_state

    def get_dup_state(self):
        return self.options['dup_state']

    def set_dynamicinterval(self, dynamicinterval):
        self.options['dynamicinterval'] = dynamicinterval

    def get_dynamicinterval(self):
        return self.options['dynamicinterval']

    def set_dynamicresponsetimeout(self, dynamicresponsetimeout):
        self.options['dynamicresponsetimeout'] = dynamicresponsetimeout

    def get_dynamicresponsetimeout(self):
        return self.options['dynamicresponsetimeout']

    def set_failureretries(self, failureretries):
        self.options['failureretries'] = failureretries

    def get_failureretries(self):
        return self.options['failureretries']

    def set_firmwarerevision(self, firmwarerevision):
        self.options['firmwarerevision'] = firmwarerevision

    def get_firmwarerevision(self):
        return self.options['firmwarerevision']

    def set_httprequest(self, httprequest):
        self.options['httprequest'] = httprequest

    def get_httprequest(self):
        return self.options['httprequest']

    def set_inbandsecurityid(self, inbandsecurityid):
        self.options['inbandsecurityid'] = inbandsecurityid

    def get_inbandsecurityid(self):
        return self.options['inbandsecurityid']

    def set_interval(self, interval):
        self.options['interval'] = interval

    def get_interval(self):
        return self.options['interval']

    def set_iptunnel(self, iptunnel):
        self.options['iptunnel'] = iptunnel

    def get_iptunnel(self):
        return self.options['iptunnel']

    def set_lrtm(self, lrtm):
        self.options['lrtm'] = lrtm

    def get_lrtm(self):
        return self.options['lrtm']

    def set_lrtmconf(self, lrtmconf):
        self.options['lrtmconf'] = lrtmconf

    def get_lrtmconf(self):
        return self.options['lrtmconf']

    def set_maxforwards(self, maxforwards):
        self.options['maxforwards'] = maxforwards

    def get_maxforwards(self):
        return self.options['maxforwards']

    def set_query(self, query):
        self.options['query'] = query

    def get_query(self):
        return self.options['query']

    def set_querytype(self, querytype):
        self.options['querytype'] = querytype

    def get_querytype(self):
        return self.options['querytype']

    def set_radaccounttype(self, radaccounttype):
        self.options['radaccounttype'] = radaccounttype

    def get_radaccounttype(self):
        return self.options['radaccounttype']

    def set_radframedip(self, radframedip):
        self.options['radframedip'] = radframedip

    def get_radframedip(self):
        return self.options['radframedip']

    def set_radnasip(self, radnasip):
        self.options['radnasip'] = radnasip

    def get_radnasip(self):
        return self.options['radnasip']

    def set_recv(self, recv):
        self.options['recv'] = recv

    def get_recv(self):
        return self.options['recv']

    def set_respcode(self, respcode):
        self.options['respcode'] = respcode

    def get_respcode(self):
        return self.options['respcode']

    def set_resptimeout(self, resptimeout):
        self.options['resptimeout'] = resptimeout

    def get_resptimeout(self):
        return self.options['resptimeout']

    def set_resptimeoutthresh(self, resptimeoutthresh):
        self.options['resptimeoutthresh'] = resptimeoutthresh

    def get_resptimeoutthresh(self):
        return self.options['resptimeoutthresh']

    def set_retries(self, retries):
        self.options['retries'] = retries

    def get_retries(self):
        return self.options['retries']

    def set_reverse(self, reverse):
        self.options['reverse'] = reverse

    def get_reverse(self):
        return self.options['reverse']

    def set_rtsprequest(self, rtsprequest):
        self.options['rtsprequest'] = rtsprequest

    def get_rtsprequest(self):
        return self.options['rtsprequest']

    def set_secure(self, secure):
        self.options['secure'] = secure

    def get_secure(self):
        return self.options['secure']

    def set_send(self, send):
        self.options['send'] = send

    def get_send(self):
        return self.options['send']

    def set_sipmethod(self, sipmethod):
        self.options['sipmethod'] = sipmethod

    def get_sipmethod(self):
        return self.options['sipmethod']

    def set_snmpversion(self, snmpversion):
        self.options['snmpversion'] = snmpversion

    def get_snmpversion(self):
        return self.options['snmpversion']

    def set_state(self, state):
        self.options['state'] = state

    def get_state(self):
        return self.options['state']

    def set_storedb(self, storedb):
        self.options['storedb'] = storedb

    def get_storedb(self):
        return self.options['storedb']

    def set_storefrontacctservice(self, storefrontacctservice):
        self.options['storefrontacctservice'] = storefrontacctservice

    def get_storefrontacctservice(self):
        return self.options['storefrontacctservice']

    def set_successretries(self, successretries):
        self.options['successretries'] = successretries

    def get_successretries(self):
        return self.options['successretries']

    def set_tos(self, tos):
        self.options['tos'] = tos

    def get_tos(self):
        return self.options['tos']

    def set_transparent(self, transparent):
        self.options['transparent'] = transparent

    def get_transparent(self):
        return self.options['transparent']

    def set_type(self, lbmontype):
        self.options['type'] = lbmontype

    def get_type(self):
        return self.options['type']

    def set_units1(self, units1):
        self.options['units1'] = units1

    def get_units1(self):
        return self.options['units1']

    def set_units2(self, units2):
        self.options['units2'] = units2

    def get_units2(self):
        return self.options['units2']

    def set_units3(self, units3):
        self.options['units3'] = units3

    def get_units3(self):
        return self.options['units3']

    def set_units4(self, units4):
        self.options['units4'] = units4

    def get_units4(self):
        return self.options['units4']

    def set_validatecred(self, validatecred):
        self.options['validatecred'] = validatecred

    def get_validatecred(self):
        return self.options['validatecred']

    def set_vendorid(self, vendorid):
        self.options['vendorid'] = vendorid

    def get_vendorid(self):
        return self.options['vendorid']

    @staticmethod
    def get(nitro, monitor):
        """
        Use this API to fetch LB Monitor of given name.
        """
        __monitor = NSLBMonitor()
        __monitor.set_monitorname(monitor.get_monitorname())
        __monitor.get_resource(nitro, object_name=__monitor.get_monitorname())
        return __monitor

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured monitor resources.
        """

        __url = nitro.get_url() + NSLBMonitor.get_resourcetype()
        __json_monitors = nitro.get(__url).get_response_field(
            NSLBMonitor.get_resourcetype())
        __monitors = []
        for json_monitor in __json_monitors:
            __monitors.append(NSLBMonitor(json_monitor))
        return __monitors

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add a new LB monitor.
        """
        __resource = NSLBMonitor()
        __resource.set_monitorname(resource.get_monitorname())
        __resource.set_alertretries(resource.get_alertretries())
        __resource.set_destip(resource.get_destip())
        __resource.set_destport(resource.get_destport())
        __resource.set_deviation(resource.get_deviation())
        __resource.set_dispatcherip(resource.get_dispatcherip())
        __resource.set_dispatcherport(resource.get_dispatcherport())
        __resource.set_downtime(resource.get_downtime())
        __resource.set_dup_state(resource.get_dup_state())
        __resource.set_dynamicinterval(resource.get_dynamicinterval())
        __resource.set_dynamicresponsetimeout(
            resource.get_dynamicresponsetimeout())
        __resource.set_failureretries(resource.get_failureretries())
        __resource.set_firmwarerevision(
            resource.get_firmwarerevision())
        __resource.set_httprequest(resource.get_httprequest())
        __resource.set_inbandsecurityid(
            resource.get_inbandsecurityid())
        __resource.set_interval(resource.get_interval())
        __resource.set_iptunnel(resource.get_iptunnel())
        __resource.set_lrtm(resource.get_lrtm())
        __resource.set_lrtmconf(resource.get_lrtmconf())
        __resource.set_maxforwards(resource.get_maxforwards())
        __resource.set_query(resource.get_query())
        __resource.set_querytype(resource.get_querytype())
        __resource.set_radaccounttype(resource.get_radaccounttype())
        __resource.set_radframedip(resource.get_radframedip())
        __resource.set_radnasip(resource.get_radnasip())
        __resource.set_recv(resource.get_recv())
        __resource.set_respcode(resource.get_respcode())
        __resource.set_resptimeout(resource.get_resptimeout())
        __resource.set_resptimeoutthresh(
            resource.get_resptimeoutthresh())
        __resource.set_retries(resource.get_retries())
        __resource.set_reverse(resource.get_reverse())
        __resource.set_secure(resource.get_secure())
        __resource.set_send(resource.get_send())
        __resource.set_sipmethod(resource.get_sipmethod())
        __resource.set_snmpversion(resource.get_snmpversion())
        __resource.set_state(resource.get_state())
        __resource.set_storedb(resource.get_storedb())
        __resource.set_storefrontacctservice(
            resource.get_storefrontacctservice())
        __resource.set_successretries(resource.get_successretries())
        __resource.set_tos(resource.get_tos())
        __resource.set_transparent(resource.get_transparent())
        __resource.set_type(resource.get_type())
        __resource.set_units1(resource.get_units1())
        __resource.set_units2(resource.get_units2())
        __resource.set_units3(resource.get_units3())
        __resource.set_units4(resource.get_units4())
        __resource.set_validatecred(resource.get_validatecred())
        __resource.set_vendorid(resource.get_vendorid())
        return __resource.add_resource(nitro)

    @staticmethod
    def update(nitro, resource):
        """
        Use this API to update an existing LB monitor.
        """
        __resource = NSLBMonitor()
        __resource.set_monitorname(resource.get_monitorname())
        __resource.set_alertretries(resource.get_alertretries())
        __resource.set_destip(resource.get_destip())
        __resource.set_destport(resource.get_destport())
        __resource.set_deviation(resource.get_deviation())
        __resource.set_dispatcherip(resource.get_dispatcherip())
        __resource.set_dispatcherport(resource.get_dispatcherport())
        __resource.set_downtime(resource.get_downtime())
        __resource.set_dup_state(resource.get_dup_state())
        __resource.set_dynamicinterval(resource.get_dynamicinterval())
        __resource.set_dynamicresponsetimeout(
            resource.get_dynamicresponsetimeout())
        __resource.set_failureretries(resource.get_failureretries())
        __resource.set_firmwarerevision(
            resource.get_firmwarerevision())
        __resource.set_httprequest(resource.get_httprequest())
        __resource.set_inbandsecurityid(
            resource.get_inbandsecurityid())
        __resource.set_interval(resource.get_interval())
        __resource.set_iptunnel(resource.get_iptunnel())
        __resource.set_lrtm(resource.get_lrtm())
        __resource.set_lrtmconf(resource.get_lrtmconf())
        __resource.set_maxforwards(resource.get_maxforwards())
        __resource.set_query(resource.get_query())
        __resource.set_querytype(resource.get_querytype())
        __resource.set_radaccounttype(resource.get_radaccounttype())
        __resource.set_radframedip(resource.get_radframedip())
        __resource.set_radnasip(resource.get_radnasip())
        __resource.set_recv(resource.get_recv())
        __resource.set_respcode(resource.get_respcode())
        __resource.set_resptimeout(resource.get_resptimeout())
        __resource.set_resptimeoutthresh(
            resource.get_resptimeoutthresh())
        __resource.set_retries(resource.get_retries())
        __resource.set_reverse(resource.get_reverse())
        __resource.set_secure(resource.get_secure())
        __resource.set_send(resource.get_send())
        __resource.set_sipmethod(resource.get_sipmethod())
        __resource.set_snmpversion(resource.get_snmpversion())
        __resource.set_state(resource.get_state())
        __resource.set_storedb(resource.get_storedb())
        __resource.set_storefrontacctservice(
            resource.get_storefrontacctservice())
        __resource.set_successretries(resource.get_successretries())
        __resource.set_tos(resource.get_tos())
        __resource.set_transparent(resource.get_transparent())
        __resource.set_type(resource.get_type())
        __resource.set_units1(resource.get_units1())
        __resource.set_units2(resource.get_units2())
        __resource.set_units3(resource.get_units3())
        __resource.set_units4(resource.get_units4())
        __resource.set_validatecred(resource.get_validatecred())
        __resource.set_vendorid(resource.get_vendorid())
        return __resource.update_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete given lbmonitor.
        """
        __resource = NSLBMonitor()
        __resource.set_monitorname(resource.get_monitorname())
        __resource.set_type(resource.get_type())
        nsresponse = __resource.delete_resource(
            nitro, object_name=__resource.get_monitorname())
        return nsresponse



from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'


class NSRewritePolicyCSVServerBinding(NSBaseResource):

    # Binding class showing the csvserver that can be bound to rewritepolicy

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSRewritePolicyCSVServerBinding, self).__init__()
        self.options = {
            'boundto': '',
            'priority': '',
            'activepolicy': '',
            'gotopriorityexpression': '',
            'labeltype': '',
            'labelname': '',
            'name': '',
        }

        self.resourcetype = NSRewritePolicyCSVServerBinding.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "rewritepolicy_csvserver_binding"

    def set_boundto(self, boundto):
        """
        Location where policy is bound.

        Default value: 0
        """
        self.options['boundto'] = boundto

    def get_boundto(self):
        """
        Location where policy is bound.

        Default value: 0
        """
        return self.options['boundto']

    def get_priority(self):
        """
        Specifies the priority of the policy.

        Default value: 0
        """
        return self.options['priority']

    def get_activepolicy(self):
        """
        Indicates whether policy is bound or not.

        Default value: 0
        """
        return self.options['activepolicy']

    def get_gotopriorityexpression(self):
        """
        Expression specifying the priority of the next policy which will get evaluated if the current policy
        rule evaluates to TRUE.

        Default value: 0
        """
        return self.options['gotopriorityexpression']

    def get_labeltype(self):
        """
        Type of policy label invocation.

        Default value: 0
        """
        return self.options['labeltype']

    def get_labelname(self):
        """
        Name of the label to invoke if the current policy rule evaluates to TRUE.

        Default value: 0
        """
        return self.options['labelname']

    def set_name(self, name):
        """
        Name of the rewrite policy.

        Default value: 0
        """
        self.options['name'] = name

    def get_name(self):
        """
        Name of the rewrite policy.

        Default value: 0
        """
        return self.options['name']

    @staticmethod
    def get(nitro, rewritepolicy_csvserver_binding):
        """
        Use this API to fetch all configured rewritepolicy_csvserver_binding resources.
        """
        __url = nitro.get_url() + NSRewritePolicyCSVServerBinding.get_resourcetype() + "/" + rewritepolicy_csvserver_binding.get_name()
        __json_resources = nitro.get(__url).get_response_field(NSRewritePolicyCSVServerBinding.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSRewritePolicyCSVServerBinding(json_resource))
        return __resources



from nsbaseresource import NSBaseResource

__author__ = 'timl'


class NSResponderPolicy(NSBaseResource):

    # Configuration for content-switching policy resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSResponderPolicy, self).__init__()
        self.options = {
            'name': '',
            'rule': '',
            'action': '',
            'hits': '',
            'undefhits': '',
            'newname': '',
            'activepolicy': '',
            'priority': '',
        }

        self.resourcetype = NSResponderPolicy.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "responderpolicy"

    def set_name(self, name):
        """
        Name of the responder policy.

        Default value: 0
        """
        self.options['name'] = name

    def get_name(self):
        """
        Name of the responder policy.

        Default value: 0
        """
        return self.options['name']

    def set_rule(self, rule):
        """
        Expression to be used by responder policy. It has to be a boolean PI rule expression.

        Default value: 0
        """
        self.options['rule'] = rule

    def get_rule(self):
        """
        Expression to be used by responder policy. It has to be a boolean PI rule expression.

        Default value: 0
        """
        return self.options['rule']

    def set_action(self, action):
        """
        Responder action to be used by the policy.

        Default value: 0
        """
        self.options['action'] = action

    def get_action(self):
        """
        Responder action to be used by the policy.

        Default value: 0
        """
        return self.options['action']

    def set_newname(self, newname):
        """
        The new name of the responder policy.

        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = newname

    def get_newname(self):
        """
        The new name of the responder policy.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    def get_hits(self):
        """
        Number of hits.

        Default value: 0
        """
        return self.options['hits']

    def get_undefhits(self):
        """
        Number of undef hits.

        Default value: 0
        """
        return self.options['undefhits']

    def get_activepolicy(self):
        """
        Policy is in use.

        Default value: 0
        """
        return self.options['activepolicy']

    def set_priority(self, priority):
        """
        Set the priority for the responder policy.

        Default value: 0
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        Priority of the responder policy.

        Default value: 0
        """
        return self.options['priority']

    # Operations methods
    @staticmethod
    def get(nitro, responderpolicy):
        """
        Use this API to fetch responderpolicy resource of given name.
        """
        __responderpolicy = NSResponderPolicy()
        __responderpolicy.set_name(responderpolicy.get_name())
        __responderpolicy.get_resource(nitro)
        return __responderpolicy

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured responderpolicy resources.
        """
        __url = nitro.get_url() + NSResponderPolicy.get_resourcetype()
        __json_cspolicies = nitro.get(__url).get_response_field(NSResponderPolicy.get_resourcetype())
        __responderpolicies = []
        for json_responderpolicy in __json_cspolicies:
            __responderpolicies.append(NSResponderPolicy(json_responderpolicy))
        return __responderpolicies

    @staticmethod
    def add(nitro, responderpolicy):
        """
        Use this API to add responderpolicy.
        """
        __responderpolicy = NSResponderPolicy()
        __responderpolicy.set_name(responderpolicy.get_name())
        __responderpolicy.set_rule(responderpolicy.get_rule())
        __responderpolicy.set_action(responderpolicy.get_action())
        __responderpolicy.set_priority(responderpolicy.get_priority())
        return __responderpolicy.add_resource(nitro)

    @staticmethod
    def delete(nitro, responderpolicy):
        """
        Use this API to delete responderpolicy of a given name.
        """
        __responderpolicy = NSResponderPolicy()
        __responderpolicy.set_name(responderpolicy.get_name())
        nsresponse = __responderpolicy.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, responderpolicy):
        """
        Use this API to update a responderpolicy of a given name.
        """
        __responderpolicy = NSResponderPolicy()
        __responderpolicy.set_name(responderpolicy.get_name())
        __responderpolicy.set_rule(responderpolicy.get_rule())
        __responderpolicy.set_action(responderpolicy.get_action())
        __responderpolicy.set_priority(responderpolicy.get_priority())
        return __responderpolicy.update_resource(nitro)

    # No unset functionality for now.
    @staticmethod
    def rename(nitro, responderpolicy):
        """
        Use this API to rename responderpolicy.
        """
        __responderpolicy = NSResponderPolicy()
        __responderpolicy.set_name(responderpolicy.get_name())
        __responderpolicy.set_newname(responderpolicy.get_newname())
        return __responderpolicy.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLPolicy(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLPolicy, self).__init__()
        self.options = {'name': '',
                        'rule': '',
                        'reqaction': '',
                        'action': '',
                        'hits': ''}

        self.resourcetype = NSSSLPolicy.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "sslpolicy"

    # Read/write properties
    def set_name(self, name):
        """
        The name for the new SSL policy. Maximum Length: 32.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        """
        The name for the new SSL policy. Maximum Length: 32.
        Minimum length = 1
        """
        return self.options['name']

    def set_rule(self, rule):
        """
        The expression that sets the condition for application of the SSL policy. Maximum Length:1500.
        """
        self.options['rule'] = rule

    def get_rule(self):
        """
        The expression that sets the condition for application of the SSL policy. Maximum Length:1500.
        """
        return self.options['rule']

    def set_reqaction(self, reqaction):
        """
        The name of the action to be performed on the request. Refer to 'add ssl action' command to add a new action.
        Minimum length = 1
        """
        self.options['reqaction'] = reqaction

    def get_reqaction(self):
        """
        The name of the action to be performed on the request. Refer to 'add ssl action' command to add a new action.
        Minimum length = 1
        """
        return self.options['reqaction']

    # Read only properties
    def get_action(self):
        """
        The name of the action to be performed on the request.
        """
        return self.options['action']

    def get_hits(self):
        """
        Number of hits for this policy.
        """
        return self.options['hits']

    @staticmethod
    def get(nitro, sslpolicy):
        """
        Use this API to fetch sslpolicy resource of given name.
        """
        __sslpolicy = NSSSLPolicy()
        __sslpolicy.get_resource(nitro, sslpolicy.get_name())
        return __sslpolicy

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured sslpolicy resources.
        """
        __url = nitro.get_url() + NSSSLPolicy.get_resourcetype()
        __json_sslpolicies = nitro.get(__url).get_response_field(NSSSLPolicy.get_resourcetype())
        __sslpolicies = []
        for json_sslpolicy in __json_sslpolicies:
            __sslpolicies.append(NSSSLPolicy(json_sslpolicy))
        return __sslpolicies

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add resource.
        """
        __resource = NSSSLPolicy()
        __resource.set_name(resource.get_name())
        __resource.set_rule(resource.get_rule())
        __resource.set_reqaction(resource.get_reqaction())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete server of a given name.
        """
        __resource = NSSSLPolicy()
        __resource.set_name(resource.get_name())
        return __resource.delete_resource(nitro)


from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLVServerSSLPolicyBinding(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLVServerSSLPolicyBinding, self).__init__()
        self.options = {'priority': '',
                        'policyname': '',
                        'vservername': '',
                        'polinherit': ''}

        self.resourcetype = NSSSLVServerSSLPolicyBinding.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "sslvserver_sslpolicy_binding"

    # Read/write properties
    def set_priority(self, priority):
        """
        The priority of the policies bound to this SSL service.
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        The priority of the policies bound to this SSL service.
        """
        return self.options['priority']

    def set_policyname(self, policyname):
        """
        The name of the SSL policy binding.
        """
        self.options['policyname'] = policyname

    def get_policyname(self):
        """
        The name of the SSL policy binding.
        """
        return self.options['policyname']

    def set_vservername(self, vservername):
        """
        The name of the SSL virtual server to which the SSL policy needs to be bound.
        Minimum length = 1
        """
        self.options['vservername'] = vservername

    def get_vservername(self):
        """
        The name of the SSL virtual server to which the SSL policy needs to be bound.
        Minimum length = 1
        """
        return self.options['vservername']

    # Read only properties
    def get_polinherit(self):
        """
        Whether the bound policy is a inherited policy or not.
        """
        return self.options['polinherit']

    @staticmethod
    def get(nitro, resource):
        """
        Use this API to fetch sslvserver resource of given name.
        """
        __url = nitro.get_url() + NSSSLVServerSSLPolicyBinding.get_resourcetype() + \
            "/" + resource.get_vservername()
        __json_resources = nitro.get(__url).get_response_field(
            NSSSLVServerSSLPolicyBinding.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSSSLVServerSSLPolicyBinding(json_resource))
        return __resources

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add resource.
        """
        __resource = NSSSLVServerSSLPolicyBinding()
        __resource.set_vservername(resource.get_vservername())
        __resource.set_policyname(resource.get_policyname())
        __resource.set_priority(resource.get_priority())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete server of a given name.
        """
        __resource = NSSSLVServerSSLPolicyBinding()
        __resource.set_vservername(resource.get_vservername())
        return __resource.delete_resource(nitro)



from nsbaseresource import NSBaseResource

__author__ = 'ivanxx@gmail.com'


class SNMPCommunity(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(SNMPCommunity, self).__init__()

        self.options = {'communityname': '',
                        'permissions': ''}

        self.resourcetype = SNMPCommunity.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "snmpcommunity"

    def set_communityname(self, communityname):
        self.options['communityname'] = communityname

    def set_permissions(self, permissions):
        self.options['permissions'] = permissions

    def get_communityname(self):
        return self.options['communityname']

    def get_permissions(self):
        return self.options['permissions']

    @staticmethod
    def add(nitro, snmpcommunity):
        __snmpcommunity = SNMPCommunity()
        __snmpcommunity.set_communityname(snmpcommunity.get_communityname())
        __snmpcommunity.set_permissions(snmpcommunity.get_permissions())
        return __snmpcommunity.add_resource(nitro)

    @staticmethod
    def delete(nitro, snmpcommunity):
        __snmpcommunity = SNMPCommunity()
        __snmpcommunity.set_communityname(snmpcommunity.get_communityname())
        return __snmpcommunity.delete_resource(nitro)

    @staticmethod
    def get(nitro, snmpcommunity):
        __snmpcommunity = SNMPCommunity()
        __snmpcommunity.set_communityname(snmpcommunity)
        __snmpcommunity.get_resource(nitro, object_name=__snmpcommunity.get_communityname())
        return __snmpcommunity

    @staticmethod
    def get_all(nitro):
        __url = nitro.get_url() + SNMPCommunity.get_resourcetype()
        __json_snmpcommunities = nitro.get(__url).get_response_field(SNMPCommunity.get_resourcetype())
        __snmpcommunities = []
        for json_snmpcommunity in __json_snmpcommunities:
            __snmpcommunities.append(SNMPCommunity(json_snmpcommunity))
        return __snmpcommunities



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSCSVServer(NSBaseResource):

    # Configuration for Content Switching Virtual Server resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSCSVServer, self).__init__()

        self.options = {'name': '',
                        'servicetype': '',
                        'ipv46': '',
                        'ippattern': '',
                        'ipmask': '',
                        'range': '',
                        'port': '',
                        'state': '',
                        'stateupdate': '',
                        'cacheable': '',
                        'redirecturl': '',
                        'clttimeout': '',
                        'precedence': '',
                        'casesensitive': '',
                        'somethod': '',
                        'sopersistence': '',
                        'sopersistencetimeout': '',
                        'sothreshold': '',
                        'redirectportrewrite': '',
                        'downstateflush': '',
                        'backupvserver': '',
                        'disableprimaryondown': '',
                        'insertvserveripport': '',
                        'vipheader': '',
                        'rtspnat': '',
                        'authenticationhost': '',
                        'authentication': '',
                        'listenpolicy': '',
                        'listenpriority': '',
                        'authn401': '',
                        'authnvsname': '',
                        'push': '',
                        'pushvserver': '',
                        'pushlabel': '',
                        'pushmulticlients': '',
                        'tcpprofilename': '',
                        'httpprofilename': '',
                        'comment': '',
                        'newname': '',

                        # --- Read-only options ---

                        'ip': '',
                        'value': '',
                        'type': '',
                        'curstate': '',
                        'status': '',
                        'cachetype': '',
                        'redirect': '',
                        'homepage': '',
                        'dnsvservername': '',
                        'domain': '',
                        'servicename': '',
                        'weight': '',
                        'cachevserver': '',
                        'targetvserver': '',
                        'url': '',
                        'gotopriorityexpression': '',
                        'bindpoint': '',
                        'invoke': '',
                        'labeltype': '',
                        'labelname': '',
                        'gt2gb': '',
                        'statechangetimesec': '',
                        'statechangetimemsec': '',
                        'tickssincelaststatechange': ''}

        self.resourcetype = NSCSVServer.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "csvserver"

    def set_name(self, name):
        """
        The content switching virtual server name.
        Default value: 0<
        Minimum length =  1.
        """
        self.options['name'] = name

    def get_name(self):
        """
        The content switching virtual server name.
        Default value: 0<
        Minimum length =  1.
        """
        return self.options['name']

    def set_servicetype(self, servicetype):
        """
        The service type of the virtual server.
        Default value: 0
        """
        self.options['servicetype'] = servicetype

    def get_servicetype(self):
        """
        The service type of the virtual server.
        Default value: 0
        """
        return self.options['servicetype']

    def set_ipv46(self, ipv46):
        """
        The IP address of the virtual server.
        Default value: 0
        Minimum length =  1.
        """
        self.options['ipv46'] = ipv46

    def get_ipv46(self):
        """
        The IP address of the virtual server.
        Default value: 0
        Minimum length =  1.
        """
        return self.options['ipv46']

    def set_ippattern(self, ippattern):
        """
        The IP Pattern of the virtual server.
        Default value: 0
        """
        self.options['ippattern'] = ippattern

    def get_ippattern(self):
        """
        The IP Pattern of the virtual server.
        Default value: 0
        """
        return self.options['ippattern']

    def set_ipmask(self, ipmask):
        """
        The IP Mask of the virtual server IP Pattern.
        """
        self.options['ipmask'] = ipmask

    def get_ipmask(self):
        """
        The IP Mask of the virtual server IP Pattern.
        """
        return self.options['ipmask']

    def set_range(self, myrange):
        """
        An IP address range.
        Default value: 1
        Minimum value =  1
        """
        self.options['range'] = myrange

    def get_range(self):
        """
        An IP address range.
        Default value: 1
        Minimum value =  1
        """
        return self.options['range']

    def set_port(self, port):
        """
        A port number for the virtual server.
        Default value: 0
        Minimum value =  1
        Range 1 - 65535.
        """
        self.options['port'] = port

    def get_port(self):
        """
        A port number for the virtual server.
        Default value: 0
        Minimum value =  1
        Range 1 - 65535.
        """
        return self.options['port']

    def set_state(self, state):
        """
        The initial state, enabled or disabled, of the virtual server.
        Default value: ENABLED
        """
        self.options['state'] = state

    def get_state(self):
        """
        The initial state, enabled or disabled, of the virtual server.
        Default value: ENABLED
        """
        return self.options['state']

    def set_stateupdate(self, stateupdate):
        """
        To enable the state update for a CSW vserver.
        Default value: DISABLED
        """
        self.options['stateupdate'] = stateupdate

    def get_stateupdate(self):
        """
        To enable the state update for a CSW vserver.
        Default value: DISABLED
        """
        return self.options['stateupdate']

    def set_cacheable(self, cacheable):
        """
        Use this option to specify whether a virtual server, used for load balancing or content switching,
        routes requests to the cache redirection virtual server before sending it to the configured servers.
        Default value: NO
        """
        self.options['cacheable'] = cacheable

    def get_cacheable(self):
        """
        Use this option to specify whether a virtual server, used for load balancing or content switching,
        routes requests to the cache redirection virtual server before sending it to the configured servers.
        Default value: NO
        """
        return self.options['cacheable']

    def set_redirecturl(self, redirecturl):
        """
        The URL where traffic is redirected if the virtual server in the system becomes unavailable.
        You can enter up to 127 characters as the URL argument.

        WARNING!

        Make sure that the domain you specify in URL does not match the domain specified in the -d domainName
        argument of the add cs policy CLI command. If the same domain is specified in both arguments,
        the request will be continuously redirected to the same unavailable virtual server in the system  -
        then the user may not get the requested content.

        Default value: 0
        Minimum length =  1.
        """
        self.options['redirecturl'] = redirecturl

    def get_redirecturl(self):
        """
        The URL where traffic is redirected if the virtual server in the system becomes unavailable.
        You can enter up to 127 characters as the URL argument.

        WARNING!

        Make sure that the domain you specify in URL does not match the domain specified in the -d domainName
        argument of the add cs policy CLI command. If the same domain is specified in both arguments,
        the request will be continuously redirected to the same unavailable virtual server in the system  -
        then the user may not get the requested content.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['redirecturl']

    def set_clttimeout(self, clttimeout):
        """
        The timeout value in seconds for idle client connection.
        Default value: 0
        Minimum value =  0
        Maximum value =  31536000
        """
        self.options['clttimeout'] = clttimeout

    def get_clttimeout(self):
        """
        The timeout value in seconds for idle client connection.
        Default value: 0
        Minimum value =  0
        Maximum value =  31536000
        """
        return self.options['clttimeout']

    def set_precedence(self, precedence):
        """
        This sets the precedence between RULE-based and URL-based policies on the content switching
        virtual server. The default precedence is RULE.
        With the precedence set to RULE, incoming requests are evaluated against the content switching policies
        created with the -rule argument (using the add cs policy CLI command).
        If none of the rules match, the URL in the request is evaluated against the content switching policies
        created with the -url argument (using the add cs policy CLI command).

        Default value: RULE
        """
        self.options['precedence'] = precedence

    def get_precedence(self):
        """
        This sets the precedence between RULE-based and URL-based policies on the content switching
        virtual server. The default precedence is RULE.
        With the precedence set to RULE, incoming requests are evaluated against the content switching policies
        created with the -rule argument (using the add cs policy CLI command).
        If none of the rules match, the URL in the request is evaluated against the content switching policies
        created with the -url argument (using the add cs policy CLI command).

        Default value: RULE
        """
        return self.options['precedence']

    def set_casesensitive(self, casesensitive):
        """
        The URL lookup case option on the content switching vserver.
        If the case sensitivity of a content switching virtual server is set to 'ON', the URLs
        /a/1.html and /A/1.HTML are treated differently, and can be switched to different targets
        with appropriate content switching policies.
        If the case sensitivity is set to 'OFF', the URLs /a/1.html and /A/1.HTML are treated the same, and
        are switched to the same target.

        Default value: ON
        """
        self.options['casesensitive'] = casesensitive

    def get_casesensitive(self):
        """
        The URL lookup case option on the content switching vserver.
        If the case sensitivity of a content switching virtual server is set to 'ON', the URLs
        /a/1.html and /A/1.HTML are treated differently, and can be switched to different targets
        with appropriate content switching policies.
        If the case sensitivity is set to 'OFF', the URLs /a/1.html and /A/1.HTML are treated the same, and
        are switched to the same target.

        Default value: ON
        """
        return self.options['casesensitive']

    def set_somethod(self, somethod):
        """
        The spillover factor based on which the traffic will be given to the backupvserver once the main virtual
        server reaches the spillover threshold.

        Default value: 0
        """
        self.options['somethod'] = somethod

    def get_somethod(self):
        """
        The spillover factor based on which the traffic will be given to the backupvserver once the main virtual
        server reaches the spillover threshold.

        Default value: 0
        """
        return self.options['somethod']

    def set_sopersistence(self, sopersistence):
        """
        The state of the spillover persistence.

        Default value: DISABLED
        """
        self.options['sopersistence'] = sopersistence

    def get_sopersistence(self):
        """
        The state of the spillover persistence.

        Default value: DISABLED
        """
        return self.options['sopersistence']

    def set_sopersistencetimeout(self, sopersistencetimeout):
        """
        Default value: 2
        Minimum value =  2
        Maximum value =  1440
        """
        self.options['sopersistencetimeout'] = sopersistencetimeout

    def get_sopersistencetimeout(self):
        """
        Default value: 2
        Minimum value =  2
        Maximum value =  1440
        """
        return self.options['sopersistencetimeout']

    def set_sothreshold(self, sothreshold):
        """
        If the spillover method is set to CONNECTION or DYNAMICCONNECTION, this value is treated as the maximum
        number of connections a lb vserver will handle before spillover takes place. If the spillover method is
        set to BANDWIDTH, this value is treated as the amount of incoming and outgoing traffic (in Kbps)
        a vserver will handle before spillover takes place.

        Default value: 0
        Minimum value =  1
        Maximum value =  4294967287
        """
        self.options['sothreshold'] = sothreshold

    def get_sothreshold(self):
        """
        If the spillover method is set to CONNECTION or DYNAMICCONNECTION, this value is treated as the maximum
        number of connections a lb vserver will handle before spillover takes place. If the spillover method is
        set to BANDWIDTH, this value is treated as the amount of incoming and outgoing traffic (in Kbps)
        a vserver will handle before spillover takes place.

        Default value: 0
        Minimum value =  1
        Maximum value =  4294967287
        """
        return self.options['sothreshold']

    def set_redirectportrewrite(self, redirectportrewrite):
        """
        Enable port rewrite while performing HTTP redirect.

        Default value: DISABLED
        """
        self.options['redirectportrewrite'] = redirectportrewrite

    def get_redirectportrewrite(self):
        """
        Enable port rewrite while performing HTTP redirect.

        Default value: DISABLED
        """
        return self.options['redirectportrewrite']

    def set_downstateflush(self, downstateflush):
        """
        Perform delayed cleanup of connections on this vserver.

        Default value: ENABLED
        """
        self.options['downstateflush'] = downstateflush

    def get_downstateflush(self):
        """
        Perform delayed cleanup of connections on this vserver.

        Default value: ENABLED
        """
        return self.options['downstateflush']

    def set_backupvserver(self, backupvserver):
        """
        The backup virtual server for content switching.

        Default value: 0
        Minimum length =  1.
        """
        self.options['backupvserver'] = backupvserver

    def get_backupvserver(self):
        """
        The backup virtual server for content switching.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['backupvserver']

    def set_disableprimaryondown(self, disableprimaryondown):
        """
        When this argument is enabled, traffic will continue reaching backup vservers even after primary
        comes UP from DOWN state.

        Default value: DISABLED
        """
        self.options['disableprimaryondown'] = disableprimaryondown

    def get_disableprimaryondown(self):
        """
        When this argument is enabled, traffic will continue reaching backup vservers even after primary
        comes UP from DOWN state.

        Default value: DISABLED
        """
        return self.options['disableprimaryondown']

    def set_insertvserveripport(self, insertvserveripport):
        """
        The virtual IP and port header insertion option for the vserver.
        VIPADDR - Header contains the vserver's IP address and port number without any translation.
        OFF - The virtual IP and port header insertion option is disabled.
        V6TOV4MAPPING - Header contains the mapped IPv4 address corresponding to the IPv6 address of the vserver
        and the port number. An IPv6 address can be mapped to a user-specified IPv4 address using
        the set ns ip6 command.

        Default value: OFF
        """
        self.options['insertvserveripport'] = insertvserveripport

    def get_insertvserveripport(self):
        """
        The virtual IP and port header insertion option for the vserver.
        VIPADDR - Header contains the vserver's IP address and port number without any translation.
        OFF - The virtual IP and port header insertion option is disabled.
        V6TOV4MAPPING - Header contains the mapped IPv4 address corresponding to the IPv6 address of the vserver
        and the port number. An IPv6 address can be mapped to a user-specified IPv4 address using
        the set ns ip6 command.

        Default value: OFF
        """
        return self.options['insertvserveripport']

    def set_vipheader(self, vipheader):
        """
        The name of virtual IP and port header.

        Default value: 0
        Minimum length =  1.
        """
        self.options['vipheader'] = vipheader

    def get_vipheader(self):
        """
        The name of virtual IP and port header.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['vipheader']

    def set_rtspnat(self, rtspnat):
        """
        Use this parameter to enable natting for RTSP data connection.

        Default value: OFF
        """
        self.options['rtspnat'] = rtspnat

    def get_rtspnat(self):
        """
        Use this parameter to enable natting for RTSP data connection.

        Default value: OFF
        """
        return self.options['rtspnat']

    def set_authenticationhost(self, authenticationhost):
        """
        FQDN of authentication vserver.

        Default value: 0
        Minimum length =  3.
        Maximum length =  252.
        """
        self.options['authenticationhost'] = authenticationhost

    def get_authenticationhost(self):
        """
        FQDN of authentication vserver.

        Default value: 0
        Minimum length =  3.
        Maximum length =  252.
        """
        return self.options['authenticationhost']

    def set_authentication(self, authentication):
        """
        This option toggles on or off the application of authentication of incoming users to the vserver.

        Default value: OFF
        """
        self.options['authentication'] = authentication

    def get_authentication(self):
        """
        This option toggles on or off the application of authentication of incoming users to the vserver.

        Default value: OFF
        """
        return self.options['authentication']

    def set_listenpolicy(self, listenpolicy):
        """
        Use this parameter to specify the listen policy for CS Vserver.
        The string can be either an existing expression name (configured using add policy expression command)
        or else it can be an in-line expression with a maximum of 1500 characters.

        Default value: "none"
        """
        self.options['listenpolicy'] = listenpolicy

    def get_listenpolicy(self):
        """
        Use this parameter to specify the listen policy for CS Vserver.
        The string can be either an existing expression name (configured using add policy expression command)
        or else it can be an in-line expression with a maximum of 1500 characters.

        Default value: "none"
        """
        return self.options['listenpolicy']

    def set_listenpriority(self, listenpriority):
        """
        Use this parameter to specify the priority for listen policy of CS Vserver.

        Default value: 101
        Minimum value =  0
        Maximum value =  100
        """
        self.options['listenpriority'] = listenpriority

    def get_listenpriority(self):
        """
        Use this parameter to specify the priority for listen policy of CS Vserver.

        Default value: 101
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['listenpriority']

    def set_authn401(self, authn401):
        """
        This option toggles on or off the HTTP 401 response based authentication.

        Default value: OFF
        """
        self.options['authn401'] = authn401

    def get_authn401(self):
        """
        This option toggles on or off the HTTP 401 response based authentication.

        Default value: OFF
        """
        return self.options['authn401']

    def set_authnvsname(self, authnvsname):
        """
        Name of authentication vserver.

        Default value: 0
        Minimum length =  1.
        Maximum length =  252.
        """
        self.options['authnvsname'] = authnvsname

    def get_authnvsname(self):
        """
        Name of authentication vserver.

        Default value: 0
        Minimum length =  1.
        Maximum length =  252.
        """
        return self.options['authnvsname']

    def set_push(self, push):
        """
        Process traffic on bound Push vserver.

        Default value: DISABLED
        """
        self.options['push'] = push

    def get_push(self):
        """
        Process traffic on bound Push vserver.

        Default value: DISABLED
        """
        return self.options['push']

    def set_pushvserver(self, pushvserver):
        """
        The lb vserver of type PUSH/SSL_PUSH to which server pushes the updates received on the client facing
        non-push lb vserver.

        Default value: 0
        Minimum length =  1.
        """
        self.options['pushvserver'] = pushvserver

    def get_pushvserver(self):
        """
        The lb vserver of type PUSH/SSL_PUSH to which server pushes the updates received on the client facing
        non-push lb vserver.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['pushvserver']

    def set_pushlabel(self, pushlabel):
        """
        Use this parameter to specify the expression to extract the label in response from server.
        The string can be either a named expression (configured using add policy expression command) or else it
        can be an in-line expression with a maximum of 63 characters.

        Default value: "none"
        """
        self.options['pushlabel'] = pushlabel

    def get_pushlabel(self):
        """
        Use this parameter to specify the expression to extract the label in response from server.
        The string can be either a named expression (configured using add policy expression command) or else it
        can be an in-line expression with a maximum of 63 characters.

        Default value: "none"
        """
        return self.options['pushlabel']

    def set_pushmulticlients(self, pushmulticlients):
        """
        Specify if multiple web 2.0 connections from the same client can connect
        to this vserver and expect updates.

        Default value: NO
        """
        self.options['pushmulticlients'] = pushmulticlients

    def get_pushmulticlients(self):
        """
        Specify if multiple web 2.0 connections from the same client can connect
        to this vserver and expect updates.

        Default value: NO
        """
        return self.options['pushmulticlients']

    def set_tcpprofilename(self, tcpprofilename):
        """
        The name of the TCP profile.

        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        self.options['tcpprofilename'] = tcpprofilename

    def get_tcpprofilename(self):
        """
        The name of the TCP profile.

        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        return self.options['tcpprofilename']

    def set_httpprofilename(self, httpprofilename):
        """
        Name of the HTTP profile.

        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        self.options['httpprofilename'] = httpprofilename

    def get_httpprofilename(self):
        """
        Name of the HTTP profile.

        Default value: 0
        Minimum length =  1.
        Maximum length =  127.
        """
        return self.options['httpprofilename']

    def set_comment(self, comment):
        """
        Comments associated with this virtual server.

        Default value: 0
        """
        self.options['comment'] = comment

    def get_comment(self):
        """
        Comments associated with this virtual server.

        Default value: 0
        """
        return self.options['comment']

    def set_newname(self, newname):
        """
        The new name of the virtual server.

        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = newname

    def get_newname(self):
        """
        The new name of the virtual server.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    def get_ip(self):
        """
        The IP address of the virtual server.

        Default value: 0
        """
        return self.options['ip']

    def get_value(self):
        """
        The ssl card status for the transparent ssl cs vserver.

        Default value: 0
        """
        return self.options['value']

    def get_type(self):
        """
        Virtual server type.

        Default value: 0
        """
        return self.options['type']

    def get_curstate(self):
        """
        The state of the cs vserver.

        Default value: 0
        """
        return self.options['curstate']

    def get_status(self):
        """
        Status.

        Default value: 0
        """
        return self.options['status']

    def get_cachetype(self):
        """
        Cache type.

        Default value: 0
        """
        return self.options['cachetype']

    def get_redirect(self):
        """
        Redirect UTL string.

        Default value: 0
        """
        return self.options['redirect']

    def get_homepage(self):
        """
        Home page.

        Default value: 0
        """
        return self.options['homepage']

    def get_dnsvservername(self):
        """
        DNS vserver name.

        Default value: 0
        """
        return self.options['dnsvservername']

    def get_domain(self):
        """
        Domain.

        Default value: 0
        """
        return self.options['domain']

    def get_servicename(self):
        """
        Service name.

        Default value: 0
        """
        return self.options['servicename']

    def get_weight(self):
        """
        Weight for this service.

        Default value: 0
        """
        return self.options['weight']

    def get_cachevserver(self):
        """
        Cache vserver name.

        Default value: 0
        """
        return self.options['cachevserver']

    def get_targetvserver(self):
        """
        Target vserver name.

        Default value: 0
        """
        return self.options['targetvserver']

    def get_url(self):
        """
        URL string.

        Default value: 0
        """
        return self.options['url']

    def get_gotopriorityexpression(self):
        """
        Expression specifying the priority of the next policy which will get evaluated if the current policy
        rule evaluates to TRUE.

        Default value: 0
        """
        return self.options['gotopriorityexpression']

    def get_bindpoint(self):
        """
        The bindpoint to which the policy is bound.

        Default value: 0
        """
        return self.options['bindpoint']

    def get_invoke(self):
        """
        Invoke flag.

        Default value: 0
        """
        return self.options['invoke']

    def get_labeltype(self):
        """
        The invocation type.

        Default value: 0
        """
        return self.options['labeltype']

    def get_labelname(self):
        """
        Name of the label invoked.

        Default value: 0
        """
        return self.options['labelname']

    def get_gt2gb(self):
        """
        This argument has no effect.

        Default value: 0
        """
        return self.options['gt2gb']

    def get_statechangetimesec(self):
        """
        Time when last state change happened. Seconds part.

        Default value: 0
        """
        return self.options['statechangetimesec']

    def get_statechangetimemsec(self):
        """
        Time when last state change happened. Milliseconds part.

        Default value: 0
        """
        return self.options['statechangetimemsec']

    def get_tickssincelaststatechange(self):
        """
        Time in 10 millisecond ticks since the last state change.

        Default value: 0
        """
        return self.options['tickssincelaststatechange']

    # Operations methods
    @staticmethod
    def disable(nitro, csvserver):
        """
        Use this API to disable csvserver.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        return __csvserver.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, csvserver):
        """
        Use this API to enable csvserver.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        return __csvserver.perform_operation(nitro, "enable")

    @staticmethod
    def rename(nitro, csvserver):
        """
        Use this API to rename csvserver.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        __csvserver.set_newname(csvserver.get_newname())
        return __csvserver.perform_operation(nitro, "rename")

    @staticmethod
    def get(nitro, csvserver):
        """
        Use this API to fetch csvserver resource of given name.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        __csvserver.get_resource(nitro)
        return __csvserver

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured csvserver resources.
        """
        __url = nitro.get_url() + NSCSVServer.get_resourcetype()
        __json_csvservers = nitro.get(__url).get_response_field(NSCSVServer.get_resourcetype())
        __csvservers = []
        for json_csvserver in __json_csvservers:
            __csvservers.append(NSCSVServer(json_csvserver))
        return __csvservers

    @staticmethod
    def add(nitro, csvserver):
        """
        Use this API to add csvserver.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        __csvserver.set_servicetype(csvserver.get_servicetype())
        __csvserver.set_ipv46(csvserver.get_ipv46())
        __csvserver.set_ippattern(csvserver.get_ippattern())
        __csvserver.set_ipmask(csvserver.get_ipmask())
        __csvserver.set_range(csvserver.get_range())
        __csvserver.set_port(csvserver.get_port())
        __csvserver.set_state(csvserver.get_state())
        __csvserver.set_stateupdate(csvserver.get_stateupdate())
        __csvserver.set_cacheable(csvserver.get_cacheable())
        __csvserver.set_redirecturl(csvserver.get_redirecturl())
        __csvserver.set_clttimeout(csvserver.get_clttimeout())
        __csvserver.set_precedence(csvserver.get_precedence())
        __csvserver.set_casesensitive(csvserver.get_casesensitive())
        __csvserver.set_somethod(csvserver.get_somethod())
        __csvserver.set_sopersistence(csvserver.get_sopersistence())
        __csvserver.set_sopersistencetimeout(csvserver.get_sopersistencetimeout())
        __csvserver.set_sothreshold(csvserver.get_sothreshold())
        __csvserver.set_redirectportrewrite(csvserver.get_redirectportrewrite())
        __csvserver.set_downstateflush(csvserver.get_downstateflush())
        __csvserver.set_backupvserver(csvserver.get_backupvserver())
        __csvserver.set_disableprimaryondown(csvserver.get_disableprimaryondown())
        __csvserver.set_insertvserveripport(csvserver.get_insertvserveripport())
        __csvserver.set_vipheader(csvserver.get_vipheader())
        __csvserver.set_rtspnat(csvserver.get_rtspnat())
        __csvserver.set_authenticationhost(csvserver.get_authenticationhost())
        __csvserver.set_authentication(csvserver.get_authentication())
        __csvserver.set_listenpolicy(csvserver.get_listenpolicy())
        __csvserver.set_listenpriority(csvserver.get_listenpriority())
        __csvserver.set_authn401(csvserver.get_authn401())
        __csvserver.set_authnvsname(csvserver.get_authnvsname())
        __csvserver.set_push(csvserver.get_push())
        __csvserver.set_pushvserver(csvserver.get_pushvserver())
        __csvserver.set_pushlabel(csvserver.get_pushlabel())
        __csvserver.set_pushmulticlients(csvserver.get_pushmulticlients())
        __csvserver.set_tcpprofilename(csvserver.get_tcpprofilename())
        __csvserver.set_httpprofilename(csvserver.get_httpprofilename())
        __csvserver.set_comment(csvserver.get_comment())
        return __csvserver.add_resource(nitro)

    @staticmethod
    def delete(nitro, csvserver):
        """
        Use this API to delete csvserver of a given name.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        nsresponse = __csvserver.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, csvserver):
        """
        Use this API to update csvserver of a given name.
        """
        __csvserver = NSCSVServer()
        __csvserver.set_name(csvserver.get_name())
        __csvserver.set_ipv46(csvserver.get_ipv46())
        __csvserver.set_ippattern(csvserver.get_ippattern())
        __csvserver.set_ipmask(csvserver.get_ipmask())
        __csvserver.set_stateupdate(csvserver.get_stateupdate())
        __csvserver.set_precedence(csvserver.get_precedence())
        __csvserver.set_casesensitive(csvserver.get_casesensitive())
        __csvserver.set_backupvserver(csvserver.get_backupvserver())
        __csvserver.set_redirecturl(csvserver.get_redirecturl())
        __csvserver.set_cacheable(csvserver.get_cacheable())
        __csvserver.set_clttimeout(csvserver.get_clttimeout())
        __csvserver.set_somethod(csvserver.get_somethod())
        __csvserver.set_sopersistence(csvserver.get_sopersistence())
        __csvserver.set_sopersistencetimeout(csvserver.get_sopersistencetimeout())
        __csvserver.set_sothreshold(csvserver.get_sothreshold())
        __csvserver.set_redirectportrewrite(csvserver.get_redirectportrewrite())
        __csvserver.set_downstateflush(csvserver.get_downstateflush())
        __csvserver.set_disableprimaryondown(csvserver.get_disableprimaryondown())
        __csvserver.set_insertvserveripport(csvserver.get_insertvserveripport())
        __csvserver.set_vipheader(csvserver.get_vipheader())
        __csvserver.set_rtspnat(csvserver.get_rtspnat())
        __csvserver.set_authenticationhost(csvserver.get_authenticationhost())
        __csvserver.set_authentication(csvserver.get_authentication())
        __csvserver.set_listenpolicy(csvserver.get_listenpolicy())
        __csvserver.set_listenpriority(csvserver.get_listenpriority())
        __csvserver.set_authn401(csvserver.get_authn401())
        __csvserver.set_authnvsname(csvserver.get_authnvsname())
        __csvserver.set_push(csvserver.get_push())
        __csvserver.set_pushvserver(csvserver.get_pushvserver())
        __csvserver.set_pushlabel(csvserver.get_pushlabel())
        __csvserver.set_pushmulticlients(csvserver.get_pushmulticlients())
        __csvserver.set_tcpprofilename(csvserver.get_tcpprofilename())
        __csvserver.set_httpprofilename(csvserver.get_httpprofilename())
        __csvserver.set_comment(csvserver.get_comment())
        return __csvserver.update_resource(nitro)



from nsbaseresource import NSBaseResource
__author__ = 'Aleksandar Topuzovic'


class NSSSLCertLink(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSSSLCertLink, self).__init__()
        self.options = {'certkeyname': '',
                        'linkcertkeyname': ''}

        self.resourcetype = NSSSLCertLink.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        """
        Binding object showing the lbmonitor that can be bound to service.
        """
        return "sslcertlink"

    # Read only properties
    def get_certkeyname(self):
        """
        Certificate key name.
        """
        return self.options['certkeyname']

    def get_linkcertkeyname(self):
        """
        Name of the Certificate-Authority.
        """
        return self.options['linkcertkeyname']

    @staticmethod
    def get(nitro, resource):
        """
        Use this API to fetch resource resource of given name.
        """
        __resource = NSSSLCertLink()
        __resource.get_resource(nitro, resource.get_name())
        return __resource

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured resource resources.
        """
        __url = nitro.get_url() + NSSSLCertLink.get_resourcetype()
        __json_resources = nitro.get(__url).get_response_field(NSSSLCertLink.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSSSLCertLink(json_resource))
        return __resources



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSConfig(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSConfig, self).__init__()

        self.options = {'force': '',
                        'level': '',
                        'ipaddress': '',
                        'netmask': '',
                        'nsvlan': '',
                        'ifnum': '',
                        'httpport': '',
                        'maxconn': '',
                        'maxreq': '',
                        'cip': '',
                        'cipheader': '',
                        'cookieversion': '',
                        'securecookie': '',
                        'pmtumin': '',
                        'pmtutimeout': '',
                        'ftpportrange': '',
                        'timezone': '',
                        'grantquotamaxclient': '',
                        'exclusivequotamaxclient': '',
                        'grantquotaspillover': '',
                        'exclusivequotaspillover': '',
                        'config1': '',
                        'config2': '',
                        'outtype': '',
                        'template': '',
                        'message': '',
                        'range': '',
                        'failover': '',
                        'primaryip': '',
                        'flags': '',
                        'lastconfigchangedtime': '',
                        'systemtime': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSConfig.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "nsconfig"

    def set_force(self, force):
        """
        Clear the configuration without prompting confirmation.

        Default value: 0
        """
        self.options['force'] = force

    def get_force(self):
        """
        Clear the configuration without prompting confirmation.

        Default value: 0
        """
        return self.options['force']

    def set_level(self, level):
        """
        Clear the configuration according to the levels.

        Default value: 0
        """
        self.options['level'] = level

    def get_level(self):
        """
        Clear the configuration according to the levels.

        Default value: 0
        """
        return self.options['level']

    def set_ipaddress(self, ipaddress):
        """
        The IP address of the system.

        Default value: 0
        Minimum length =  1.
        """
        self.options['ipaddress'] = ipaddress

    def get_ipaddress(self):
        """
        The IP address of the system.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['ipaddress']

    def set_netmask(self, netmask):
        """
        The netmask corresponding to the IP address.

        Default value: 0
        """
        self.options['netmask'] = netmask

    def get_netmask(self):
        """
        The netmask corresponding to the IP address.

        Default value: 0
        """
        return self.options['netmask']

    def set_nsvlan(self, nsvlan):
        """
        The VLAN (NSVLAN) for the subnet on which the IP resides.

        Default value: 0
        Minimum value =  2
        Maximum value =  4094
        """
        self.options['nsvlan'] = nsvlan

    def get_nsvlan(self):
        """
        The VLAN (NSVLAN) for the subnet on which the IP resides.

        Default value: 0
        Minimum value =  2
        Maximum value =  4094
        """
        return self.options['nsvlan']

    def set_ifnum(self, ifnum):
        """
        Bind the given ports to the NSVLAN.

        Default value: 0
        Minimum length =  1.
        """
        self.options['ifnum'] = ifnum

    def get_ifnum(self):
        """
        Bind the given ports to the NSVLAN.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['ifnum']

    def set_httpport(self, httpport):
        """
        The HTTP ports on the Web server. This allows the system to perform connection off-load for any client
        request that has a destination port matching one of these configured ports.

        Default value: 0
        Minimum value =  1
        """
        self.options['httpport'] = httpport

    def get_httpport(self):
        """
        The HTTP ports on the Web server. This allows the system to perform connection off-load for any client
        request that has a destination port matching one of these configured ports.

        Default value: 0
        Minimum value =  1
        """
        return self.options['httpport']

    def set_maxconn(self, maxconn):
        """
        The maximum number of connections that will be made from the system to the web server(s) attached to it.
        The value entered here is applied globally to all attached servers.

        Default value: 0
        Minimum value =  0
        Maximum value =  4294967294
        """
        self.options['maxconn'] = maxconn

    def get_maxconn(self):
        """
        The maximum number of connections that will be made from the system to the web server(s) attached to it.
        The value entered here is applied globally to all attached servers.

        Default value: 0
        Minimum value =  0
        Maximum value =  4294967294
        """
        return self.options['maxconn']

    def set_maxreq(self, maxreq):
        """
        The maximum number of requests that the system can pass on a particular connection between the system
        and a server attached to it. Setting this value to 0 allows an unlimited number of requests to be
        passed.

        Default value: 0
        Minimum value =  0
        Maximum value =  65535
        """
        self.options['maxreq'] = maxreq

    def get_maxreq(self):
        """
        The maximum number of requests that the system can pass on a particular connection between the system
        and a server attached to it. Setting this value to 0 allows an unlimited number of requests to be
        passed.

        Default value: 0
        Minimum value =  0
        Maximum value =  65535
        """
        return self.options['maxreq']

    def set_cip(self, cip):
        """
        The option to control (enable or disable) the insertion of the actual client IP address into the HTTP
        header request passed from the client to one, some, or all servers attached to the system.

        The passed address can then be accessed through a minor modification to the server.
                If cipHeader is specified, it will be used as the client IP header.

                If it is not specified, then the value that has been set by the set ns config CLI command will
                be used as the client IP header.

        Default value: 0
        """
        self.options['cip'] = cip

    def get_cip(self):
        """
        The option to control (enable or disable) the insertion of the actual client IP address into the HTTP
        header request passed from the client to one, some, or all servers attached to the system.

        The passed address can then be accessed through a minor modification to the server.
                If cipHeader is specified, it will be used as the client IP header.

                If it is not specified, then the value that has been set by the set ns config CLI command will
                be used as the client IP header.

        Default value: 0
        """
        return self.options['cip']

    def set_cipheader(self, cipheader):
        """
        The text that will be used as the client IP header.

        Default value: 0
        Minimum length =  1.
        """
        self.options['cipheader'] = cipheader

    def get_cipheader(self):
        """
        The text that will be used as the client IP header.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['cipheader']

    def set_cookieversion(self, cookieversion):
        """
        The version of the cookie inserted by system.

        Default value: 0
        """
        self.options['cookieversion'] = cookieversion

    def get_cookieversion(self):
        """
        The version of the cookie inserted by system.

        Default value: 0
        """
        return self.options['cookieversion']

    def set_securecookie(self, securecookie):
        """
        enable/disable secure flag for persistence cookie.

        Default value: ENABLED
        """
        self.options['securecookie'] = securecookie

    def get_securecookie(self):
        """
        enable/disable secure flag for persistence cookie.

        Default value: ENABLED
        """
        return self.options['securecookie']

    def set_pmtumin(self, pmtumin):
        """
        The minimum Path MTU.

        Default value: 576
        Minimum value =  168
        Maximum value =  1500
        """
        self.options['pmtumin'] = pmtumin

    def get_pmtumin(self):
        """
        The minimum Path MTU.

        Default value: 576
        Minimum value =  168
        Maximum value =  1500
        """
        return self.options['pmtumin']

    def set_pmtutimeout(self, pmtutimeout):
        """
        The timeout value in minutes.

        Default value: 10
        Minimum value =  1
        Maximum value =  1440
        """
        self.options['pmtutimeout'] = pmtutimeout

    def get_pmtutimeout(self):
        """
        The timeout value in minutes.

        Default value: 10
        Minimum value =  1
        Maximum value =  1440
        """
        return self.options['pmtutimeout']

    def set_ftpportrange(self, ftpportrange):
        """
        Port range configured for FTP services.

        Default value: 0
        Minimum length =  1024.
        Maximum length =  64000.
        """
        self.options['ftpportrange'] = ftpportrange

    def get_ftpportrange(self):
        """
        Port range configured for FTP services.

        Default value: 0
        Minimum length =  1024.
        Maximum length =  64000.
        """
        return self.options['ftpportrange']

    def set_timezone(self, timezone):
        """
        Name of the timezone.

        Default value: 0
        """
        self.options['timezone'] = timezone

    def get_timezone(self):
        """
        Name of the timezone.

        Default value: 0
        """
        return self.options['timezone']

    def set_grantquotamaxclient(self, grantquotamaxclient):
        """
        The percentage of shared quota to be granted at a time for maxClient.

        Default value: 10
        Minimum value =  0
        Maximum value =  100
        """
        self.options['grantquotamaxclient'] = grantquotamaxclient

    def get_grantquotamaxclient(self):
        """
        The percentage of shared quota to be granted at a time for maxClient.

        Default value: 10
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['grantquotamaxclient']

    def set_exclusivequotamaxclient(self, exclusivequotamaxclient):
        """
        The percentage of maxClient to be given to PEs.

        Default value: 80
        Minimum value =  0
        Maximum value =  100
        """
        self.options['exclusivequotamaxclient'] = exclusivequotamaxclient

    def get_exclusivequotamaxclient(self):
        """
        The percentage of maxClient to be given to PEs.

        Default value: 80
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['exclusivequotamaxclient']

    def set_grantquotaspillover(self, grantquotaspillover):
        """
        The percentage of shared quota to be granted at a time for spillover.

        Default value: 10
        Minimum value =  0
        Maximum value =  100
        """
        self.options['grantquotaspillover'] = grantquotaspillover

    def get_grantquotaspillover(self):
        """
        The percentage of shared quota to be granted at a time for spillover.

        Default value: 10
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['grantquotaspillover']

    def set_exclusivequotaspillover(self, exclusivequotaspillover):
        """
        The percentage of max limit to be given to PEs.

        Default value: 80
        Minimum value =  0
        Maximum value =  100
        """
        self.options['exclusivequotaspillover'] = exclusivequotaspillover

    def get_exclusivequotaspillover(self):
        """
        The percentage of max limit to be given to PEs.

        Default value: 80
        Minimum value =  0
        Maximum value =  100
        """
        return self.options['exclusivequotaspillover']

    def set_config1(self, config1):
        """
        Config options.

        Default value: 0
        """
        self.options['config1'] = config1

    def get_config1(self):
        """
        Config options.

        Default value: 0
        """
        return self.options['config1']

    def set_config2(self, config2):
        """
        Config options.

        Default value: 0
        """
        self.options['config2'] = config2

    def get_config2(self):
        """
        Config options.

        Default value: 0
        """
        return self.options['config2']

    def set_outtype(self, outtype):
        """
        The format in which result is desired.

        Default value: 0
        """
        self.options['outtype'] = outtype

    def get_outtype(self):
        """
        The format in which result is desired.

        Default value: 0
        """
        return self.options['outtype']

    def set_template(self, template):
        """
        Enable template diff. This will only compare commands given in template file.

        Default value: 0
        """
        self.options['template'] = template

    def get_template(self):
        """
        Enable template diff. This will only compare commands given in template file.

        Default value: 0
        """
        return self.options['template']

    def get_message(self):
        """
        Default value: 0
        """
        return self.options['message']

    def get_range(self):
        """
        The range of Mapped IP addresses to be configured.

        Default value: 0
        """
        return self.options['range']

    def get_failover(self):
        """
        Standalone node.

        Default value: 0
        """
        return self.options['failover']

    def get_primaryip(self):
        """
        HA Master Node IP address.

        Default value: 0
        """
        return self.options['primaryip']

    def get_flags(self):
        """
        The flags for this entry.

        Default value: 0
        """
        return self.options['flags']

    def get_lastconfigchangedtime(self):
        """
        Time when the configuration was last modified.

        Default value: 0
        """
        return self.options['lastconfigchangedtime']

    def get_systemtime(self):
        """
        Current system time.

        Default value: 0
        """
        return self.options['systemtime']

    @staticmethod
    def save(nitro):
        """
        Use this API to save Netscaler configuration.
        """
        __config = NSConfig()
        return __config.perform_operation(nitro, "save")

    @staticmethod
    def clear(nitro, config):
        __config = NSConfig()
        __config.set_force(config.get_force())
        __config.set_level(config.get_level())
        return __config.perform_operation(nitro, "clear")

    @staticmethod
    def update(nitro, config):
        __config = NSConfig()
        __config.set_ipaddress(config.get_ipaddress())
        __config.set_netmask(config.get_netmask())
        __config.set_nsvlan(config.get_nsvlan())
        __config.set_ifnum(config.get_ifnum())
        __config.set_httpport(config.get_httpport())
        __config.set_maxconn(config.get_maxconn())
        __config.set_maxreq(config.get_maxreq())
        __config.set_cip(config.get_cip())
        __config.set_cipheader(config.get_cipheader())
        __config.set_cookieversion(config.get_cookieversion())
        __config.set_securecookie(config.get_securecookie())
        __config.set_pmtumin(config.get_pmtumin())
        __config.set_pmtutimeout(config.get_pmtutimeout())
        __config.set_ftpportrange(config.get_ftpportrange())
        __config.set_timezone(config.get_timezone())
        __config.set_grantquotamaxclient(config.get_grantquotamaxclient())
        __config.set_exclusivequotamaxclient(config.get_exclusivequotamaxclient())
        __config.set_grantquotaspillover(config.get_grantquotaspillover())
        __config.set_exclusivequotaspillover(config.get_exclusivequotaspillover())
        return __config.update_resource(nitro)

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all nsconfig resources that are configured on netscaler.
        """
        __url = nitro.get_url() + NSConfig.get_resourcetype()
        __json_configs = nitro.get(__url).get_response_field(NSConfig.get_resourcetype())
        return NSConfig(__json_configs)



from nsnitro.nsutil import NSNitroError
import urllib
import json


class NSBaseResource(object):

    options = {}
    resourcetype = False

    def __init__(self):
        self.options = {}
        self.resourcetype = False
        self.__baseaction = False

    def __str__(self):
        ret = ""
        for key, value in self.options.items():
            ret += "\t%s: \t\t%s\n" % (key, value)

        return ret

    def set_action(self, action):
        self.__baseaction = action

    def set_options(self, options):
        self.options = options

        # Filter out empty options
        self.options = dict([(k, v) for k, v in self.options.items() if v != ''])

    def get_payload(self):
        options = dict([(k, v) for k, v in self.options.items() if v != ''])
        if self.__baseaction:
            payload = {"object": json.dumps({"params": {"action": self.__baseaction}, self.resourcetype: options})}
        else:
            payload = {"object": json.dumps({self.resourcetype: options})}
        # print payload
        return payload

    def get_put_payload(self, sessionid):
        options = dict([(k, v) for k, v in self.options.items() if v != ''])
        if self.__baseaction:
            payload = {"params": {"action": self.__baseaction}, self.resourcetype: options}
        else:
            payload = {"sessionid": sessionid, self.resourcetype: options}
        # print payload
        return payload

    def get_delete_args(self):
        options = dict([(k, v) for k, v in self.options.items() if v != ''])

        args = "?args="

        for key, value in options.iteritems():
            args = "%s%s:%s%s" % (args, key, urllib.quote_plus(value) if type(value) is str else value, ",")

        args = args[:-1]  # remove last comma

        return args

    def perform_operation(self, nitro, action):
        self.set_action(action)
        response = nitro.post(self.get_payload())
        return response

    def get_resource(self, nitro, object_name=None):
        url = "%s%s/%s" % (nitro.get_url(),
                           self.resourcetype,
                           object_name if object_name else self.options['name'])
        response = nitro.get(url)

        if response.failed:
            raise NSNitroError(response.message)

        for resource in response.get_response_field(self.resourcetype):
            for k in resource.iterkeys():
                self.options[k] = resource[k]

    def add_resource(self, nitro):
        response = nitro.post(self.get_payload())
        return response

    def update_resource(self, nitro):
        response = nitro.put(self.get_put_payload(nitro.get_sessionid()))
        if response.failed:
            raise NSNitroError(response.message)
        return response

    def delete_resource(self, nitro, object_name=None):
        url = "%s%s" % (nitro.get_url(), self.resourcetype)

        if object_name is not None:
            url += "/%s" % object_name
        elif 'name' in self.options and self.options['name'] is not None:
            url += "/%s" % self.options['name']

        urlargs = self.get_delete_args()
        url += urlargs

        response = nitro.delete(url)

        if response.failed:
            raise NSNitroError(response.message)

        return response



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSIP(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSIP, self).__init__()

        self.options = {'ipaddress': '',
                        'netmask': '',
                        'type': '',
                        'arp': '',
                        'icmp': '',
                        'vserver': '',
                        'telnet': '',
                        'ftp': '',
                        'gui': '',
                        'ssh': '',
                        'snmp': '',
                        'mgmtaccess': '',
                        'restrictaccess': '',
                        'dynamicrouting': '',
                        'ospf': '',
                        'bgp': '',
                        'rip': '',
                        'hostroute': '',
                        'hostrtgw': '',
                        'metric': '',
                        'vserverrhilevel': '',
                        'ospflsatype': '',
                        'ospfarea': '',
                        'state': '',
                        'vrid': '',
                        'flags': '',
                        'ospfareaval': '',
                        'viprtadv2bsd': '',
                        'vipvsercount': '',
                        'vipvserdowncount': '',
                        'freeports': '',
                        'iptype': '',
                        'count': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

        self.resourcetype = NSIP.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "nsip"

    def set_ipaddress(self, ipaddress):
        self.options['ipaddress'] = ipaddress

    def get_ipaddress(self):
        return self.options['ipaddress']

    def set_netmask(self, netmask):
        self.options['netmask'] = netmask

    def get_netmask(self):
        return self.options['netmask']

    def set_type(self, mytype):
        self.options['type'] = mytype

    def get_type(self):
        return self.options['type']

    def set_arp(self, arp):
        self.options['arp'] = arp

    def get_arp(self):
        return self.options['arp']

    def set_icmp(self, icmp):
        self.options['icmp'] = icmp

    def get_icmp(self):
        return self.options['icmp']

    def set_vserver(self, vserver):
        self.options['vserver'] = vserver

    def get_vserver(self):
        return self.options['vserver']

    def set_telnet(self, telnet):
        self.options['telnet'] = telnet

    def get_telnet(self):
        return self.options['telnet']

    def set_ftp(self, ftp):
        self.options['ftp'] = ftp

    def get_ftp(self):
        return self.options['ftp']

    def set_gui(self, gui):
        self.options['gui'] = gui

    def get_gui(self):
        return self.options['gui']

    def set_ssh(self, ssh):
        self.options['ssh'] = ssh

    def get_ssh(self):
        return self.options['ssh']

    def set_snmp(self, snmp):
        self.options['snmp'] = snmp

    def get_snmp(self):
        return self.options['snmp']

    def set_mgmtaccess(self, mgmtaccess):
        self.options['mgmtaccess'] = mgmtaccess

    def get_mgmtaccess(self):
        return self.options['mgmtaccess']

    def set_restrictaccess(self, restrictaccess):
        self.options['restrictaccess'] = restrictaccess

    def get_restrictaccess(self):
        return self.options['restrictaccess']

    def set_dynamicrouting(self, dynamicrouting):
        self.options['dynamicrouting'] = dynamicrouting

    def get_dynamicrouting(self):
        return self.options['dynamicrouting']

    def set_ospf(self, ospf):
        self.options['ospf'] = ospf

    def get_ospf(self):
        return self.options['ospf']

    def set_bgp(self, bgp):
        self.options['bgp'] = bgp

    def get_bgp(self):
        return self.options['bgp']

    def set_rip(self, rip):
        self.options['rip'] = rip

    def get_rip(self):
        return self.options['rip']

    def set_hostroute(self, hostroute):
        self.options['hostroute'] = hostroute

    def get_hostroute(self):
        return self.options['hostroute']

    def set_hostrtgw(self, hostrtgw):
        self.options['hostrtgw'] = hostrtgw

    def get_hostrtgw(self):
        return self.options['hostrtgw']

    def set_metric(self, metric):
        self.options['metric'] = metric

    def get_metric(self):
        return self.options['metric']

    def set_vserverrhilevel(self, vserverrhilevel):
        self.options['vserverrhilevel'] = vserverrhilevel

    def get_vserverrhilevel(self):
        return self.options['vserverrhilevel']

    def set_ospflsatype(self, ospflsatype):
        self.options['ospflsatype'] = ospflsatype

    def get_ospflsatype(self):
        return self.options['ospflsatype']

    def set_ospfarea(self, ospfarea):
        self.options['ospfarea'] = ospfarea

    def get_ospfarea(self):
        return self.options['ospfarea']

    def set_state(self, state):
        self.options['state'] = state

    def get_state(self):
        return self.options['state']

    def set_vrid(self, vrid):
        self.options['vrid'] = vrid

    def get_vrid(self):
        return self.options['vrid']

    def get_flags(self):
        return self.options['flags']

    def get_ospfareaval(self):
        return self.options['ospfareaval']

    def get_viprtadv2bsd(self):
        return self.options['viprtadv2bsd']

    def get_vipvsercount(self):
        return self.options['vipvsercount']

    def get_vipvserdowncount(self):
        return self.options['vipvserdowncount']

    def get_freeports(self):
        return self.options['freeports']

    def get_iptype(self):
        return self.options['iptype']

    def get_count(self):
        return self.options['count']

    @staticmethod
    def add(nitro, ip):
        __ip = NSIP()
        __ip.set_ipaddress(ip.get_ipaddress())
        __ip.set_netmask(ip.get_netmask())
        __ip.set_type(ip.get_type())
        __ip.set_arp(ip.get_arp())
        __ip.set_icmp(ip.get_icmp())
        __ip.set_vserver(ip.get_vserver())
        __ip.set_telnet(ip.get_telnet())
        __ip.set_ftp(ip.get_ftp())
        __ip.set_gui(ip.get_gui())
        __ip.set_ssh(ip.get_ssh())
        __ip.set_snmp(ip.get_snmp())
        __ip.set_mgmtaccess(ip.get_mgmtaccess())
        __ip.set_restrictaccess(ip.get_restrictaccess())
        __ip.set_dynamicrouting(ip.get_dynamicrouting())
        __ip.set_ospf(ip.get_ospf())
        __ip.set_bgp(ip.get_bgp())
        __ip.set_rip(ip.get_rip())
        __ip.set_hostroute(ip.get_hostroute())
        __ip.set_hostrtgw(ip.get_hostrtgw())
        __ip.set_metric(ip.get_metric())
        __ip.set_vserverrhilevel(ip.get_vserverrhilevel())
        __ip.set_ospflsatype(ip.get_ospflsatype())
        __ip.set_ospfarea(ip.get_ospfarea())
        __ip.set_state(ip.get_state())
        __ip.set_vrid(ip.get_vrid())
        return __ip.add_resource(nitro)

    @staticmethod
    def update(nitro, ip):
        __ip = NSIP()
        __ip.set_ipaddress(ip.get_ipaddress())
        __ip.set_netmask(ip.get_netmask())
        __ip.set_arp(ip.get_arp())
        __ip.set_icmp(ip.get_icmp())
        __ip.set_vserver(ip.get_vserver())
        __ip.set_telnet(ip.get_telnet())
        __ip.set_ftp(ip.get_ftp())
        __ip.set_gui(ip.get_gui())
        __ip.set_ssh(ip.get_ssh())
        __ip.set_snmp(ip.get_snmp())
        __ip.set_mgmtaccess(ip.get_mgmtaccess())
        __ip.set_restrictaccess(ip.get_restrictaccess())
        __ip.set_dynamicrouting(ip.get_dynamicrouting())
        __ip.set_ospf(ip.get_ospf())
        __ip.set_bgp(ip.get_bgp())
        __ip.set_rip(ip.get_rip())
        __ip.set_hostroute(ip.get_hostroute())
        __ip.set_hostrtgw(ip.get_hostrtgw())
        __ip.set_metric(ip.get_metric())
        __ip.set_vserverrhilevel(ip.get_vserverrhilevel())
        __ip.set_ospflsatype(ip.get_ospflsatype())
        __ip.set_ospfarea(ip.get_ospfarea())
        __ip.set_vrid(ip.get_vrid())
        return __ip.update_resource(nitro)

    @staticmethod
    def delete(nitro, ip):
        __ip = NSIP()
        __ip.set_ipaddress(ip.get_ipaddress())
        nsresponse = __ip.delete_resource(nitro, object_name=ip.get_ipaddress())
        return nsresponse

    @staticmethod
    def enable(nitro, ip):
        __ip = NSIP()
        __ip.set_ipaddress(ip.get_ipaddress())
        return __ip.perform_operation(nitro, "enable")

    @staticmethod
    def disable(nitro, ip):
        __ip = NSIP()
        __ip.set_ipaddress(ip.get_ipaddress())
        return __ip.perform_operation(nitro, "disable")

    @staticmethod
    def get(nitro, ip):
        __url = nitro.get_url() + NSIP.get_resourcetype() + "/" + ip.get_ipaddress()
        return nitro.get(__url).get_response_field(NSIP.get_resourcetype())[0]

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all nsip resources that are configured on netscaler.
        """
        __url = nitro.get_url() + NSIP.get_resourcetype()
        __json_ips = nitro.get(__url).get_response_field(NSIP.get_resourcetype())
        __ips = []
        for json_ip in __json_ips:
            __ips.append(NSIP(json_ip))
        return __ips



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSSSLCertKey(NSBaseResource):

    # General Netscaler configuration object

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """

        super(NSSSLCertKey, self).__init__()

        self.options = {'certkey': '',
                        'cert': '',
                        'key': '',
                        'password': '',
                        'fipskey': '',
                        'inform': '',
                        'passplain': '',
                        'expirymonitor': '',
                        'notificationperiod': '',
                        'linkcertkeyname': '',
                        'nodomaincheck': '',
                        'signaturealg': '',
                        'serial': '',
                        'issuer': '',
                        'clientcertnotbefore': '',
                        'clientcertnotafter': '',
                        'daystoexpiration': '',
                        'subject': '',
                        'publickey': '',
                        'publickeysize': '',
                        'version': '',
                        'priority': '',
                        'status': '',
                        'passcrypt': '',
                        'data': '',
                        'servicename': ''}

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSSSLCertKey.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "sslcertkey"

    def set_certkey(self, certkey):
        self.options['certkey'] = certkey

    def get_certkey(self):
        return self.options['certkey']

    def set_cert(self, cert):
        self.options['cert'] = cert

    def get_cert(self):
        return self.options['cert']

    def set_key(self, key):
        self.options['key'] = key

    def get_key(self):
        return self.options['key']

    def set_password(self, password):
        self.options['password'] = password

    def get_password(self):
        return self.options['password']

    def set_fipskey(self, fipskey):
        self.options['fipskey'] = fipskey

    def get_fipskey(self):
        return self.options['fipskey']

    def set_inform(self, inform):
        self.options['inform'] = inform

    def get_inform(self):
        return self.options['inform']

    def set_passplain(self, passplain):
        self.options['passplain'] = passplain

    def get_passplain(self):
        return self.options['passplain']

    def set_expirymonitor(self, expirymonitor):
        self.options['expirymonitor'] = expirymonitor

    def get_expirymonitor(self):
        return self.options['expirymonitor']

    def set_notificationperiod(self, notificationperiod):
        self.options['notificationperiod'] = notificationperiod

    def get_notificationperiod(self):
        return self.options['notificationperiod']

    def set_linkcertkeyname(self, linkcertkeyname):
        self.options['linkcertkeyname'] = linkcertkeyname

    def get_linkcertkeyname(self):
        return self.options['linkcertkeyname']

    def set_nodomaincheck(self, nodomaincheck):
        self.options['nodomaincheck'] = nodomaincheck

    def get_nodomaincheck(self):
        return self.options['nodomaincheck']

    def set_passcrypt(self, passcrypt):
        self.options['passcrypt'] = passcrypt

    def get_passcrypt(self):
        return self.options['passcrypt']

    def get_signaturealg(self):
        return self.options['signaturealg']

    def get_serial(self):
        return self.options['serial']

    def get_issuer(self):
        return self.options['issuer']

    def get_clientcertnotbefore(self):
        return self.options['clientcertnotbefore']

    def get_clientcertnotafter(self):
        return self.options['clientcertnotafter']

    def get_daystoexpiration(self):
        return self.options['daystoexpiration']

    def get_subject(self):
        return self.options['subject']

    def get_publickey(self):
        return self.options['publickey']

    def get_publickeysize(self):
        return self.options['publickeysize']

    def get_version(self):
        return self.options['version']

    def get_priority(self):
        return self.options['priority']

    def get_status(self):
        return self.options['status']

    def get_data(self):
        return self.options['data']

    def get_servicename(self):
        return self.options['servicename']

    @staticmethod
    def get(nitro, resource):
        """
        Use this API to fetch SSL certkey resource of given name.
        """
        __resource = NSSSLCertKey()
        __resource.get_resource(nitro, resource.get_certkey())
        return __resource

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured SSL certkey resources.
        """
        __url = nitro.get_url() + NSSSLCertKey.get_resourcetype()
        __json_resources = nitro.get(__url).get_response_field(NSSSLCertKey.get_resourcetype())
        __resources = []
        for json_resource in __json_resources:
            __resources.append(NSSSLCertKey(json_resource))
        return __resources

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add resource.
        """
        __resource = NSSSLCertKey()
        __resource.set_certkey(resource.get_certkey())
        __resource.set_cert(resource.get_cert())
        __resource.set_key(resource.get_key())
        __resource.set_password(resource.get_password())
        __resource.set_fipskey(resource.get_fipskey())
        __resource.set_inform(resource.get_inform())
        __resource.set_passplain(resource.get_passplain())
        __resource.set_passcrypt(resource.get_passcrypt())
        __resource.set_expirymonitor(resource.get_expirymonitor())
        __resource.set_notificationperiod(resource.get_notificationperiod())
        return __resource.add_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete server of a given name.
        """
        __resource = NSSSLCertKey()
        __resource.set_certkey(resource.get_certkey())
        nsresponse = __resource.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, resource):
        """
        Use this API to update a server of a given name.
        """
        __resource = NSSSLCertKey()
        __resource.set_certkey(resource.get_certkey())
        __resource.set_expirymonitor(resource.get_expirymonitor())
        __resource.set_notificationperiod(resource.get_notificationperiod())
        return __resource.update_resource(nitro)

    @staticmethod
    def link(nitro, resource):
        """
        Use this API to link resource of a given name
        """
        __resource = NSSSLCertKey()
        __resource.set_certkey(resource.get_certkey())
        __resource.set_linkcertkeyname(resource.get_linkcertkeyname())
        nsresponse = __resource.perform_operation(nitro, "link")
        return nsresponse

    @staticmethod
    def unlink(nitro, resource):
        """
        Use this API to link resource of a given name
        """
        __resource = NSSSLCertKey()
        __resource.set_certkey(resource.get_certkey())
        nsresponse = __resource.perform_operation(nitro, "unlink")
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSServiceGroup(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSServiceGroup, self).__init__()

        self.options = {
            'servicegroupname': '',
            'servicetype': '',
            'cachetype': '',
            'maxclient': '',
            'maxreq': '',
            'cacheable': '',
            'cip': '',
            'cipheader': '',
            'usip': '',
            'useproxyport': '',
            'sc': '',
            'sp': '',
            'rtspsessionidremap': '',
            'clttimeout': '',
            'svrtimeout': '',
            'cka': '',
            'tcpb': '',
            'cmp': '',
            'maxbandwidth': '',
            'monthreshold': '',
            'state': '',
            'downstateflush': '',
            'tcpprofilename': '',
            'httpprofilename': '',
            'comment': '',
            'servername': '',
            'port': '',
            'weight': '',
            'serverid': '',
            'monitor_name_svc': '',
            'dup_weight': '',
            'delay': '',
            'graceful': '',
            'includemembers': '',
            'newname': '',
            'serviceconftype': '',
            'value': '',
            'svrstate': '',
            'ip': '',
            'monitostate': '',
            'monstatcode': '',
            'monstatparam1': '',
            'monstatparam2': '',
            'monstatparam3': '',
            'statechangetimesec': '',
            'statechangetimemsec': '',
            'timesincelaststatechange': '',
            'tickssincelaststatechange': '',
            'stateupdatereason': '',
            'groupcount': '',
        }

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options:
                    self.options[key] = json_data[key]

        self.resourcetype = NSServiceGroup.get_resourcetype()

    @staticmethod
    def get_resourcetype():
        return "servicegroup"

    def set_servicegroupname(self, servicegroupname):
        self.options['servicegroupname'] = servicegroupname

    def get_servicegroupname(self):
        return self.options['servicegroupname']

    def set_servicetype(self, servicetype):
        self.options['servicetype'] = servicetype

    def get_servicetype(self):
        return self.options['servicetype']

    def set_cachetype(self, cachetype):
        self.options['cachetype'] = cachetype

    def get_cachetype(self):
        return self.options['cachetype']

    def set_maxclient(self, maxclient):
        self.options['maxclient'] = maxclient

    def get_maxclient(self):
        return self.options['maxclient']

    def set_maxreq(self, maxreq):
        self.options['maxreq'] = maxreq

    def get_maxreq(self):
        return self.options['maxreq']

    def set_cacheable(self, cacheable):
        self.options['cacheable'] = cacheable

    def get_cacheable(self):
        return self.options['cacheable']

    def set_cip(self, cip):
        self.options['cip'] = cip

    def get_cip(self):
        return self.options['cip']

    def set_cipheader(self, cipheader):
        self.options['cipheader'] = cipheader

    def get_cipheader(self):
        return self.options['cipheader']

    def set_usip(self, usip):
        self.options['usip'] = usip

    def get_usip(self):
        return self.options['usip']

    def set_useproxyport(self, useproxyport):
        self.options['useproxyport'] = useproxyport

    def get_useproxyport(self):
        return self.options['useproxyport']

    def set_sc(self, sc):
        self.options['sc'] = sc

    def get_sc(self):
        return self.options['sc']

    def set_sp(self, sp):
        self.options['sp'] = sp

    def get_sp(self):
        return self.options['sp']

    def set_rtspsessionidremap(self, rtspsessionidremap):
        self.options['rtspsessionidremap'] = rtspsessionidremap

    def get_rtspsessionidremap(self):
        return self.options['rtspsessionidremap']

    def set_clttimeout(self, clttimeout):
        self.options['clttimeout'] = clttimeout

    def get_clttimeout(self):
        return self.options['clttimeout']

    def set_svrtimeout(self, svrtimeout):
        self.options['svrtimeout'] = svrtimeout

    def get_svrtimeout(self):
        return self.options['svrtimeout']

    def set_cka(self, cka):
        self.options['cka'] = cka

    def get_cka(self):
        return self.options['cka']

    def set_tcpb(self, tcpb):
        self.options['tcpb'] = tcpb

    def get_tcpb(self):
        return self.options['tcpb']

    def set_cmp(self, scmp):
        self.options['cmp'] = scmp

    def get_cmp(self):
        return self.options['cmp']

    def set_maxbandwidth(self, maxbandwidth):
        self.options['maxbandwidth'] = maxbandwidth

    def get_maxbandwidth(self):
        return self.options['maxbandwidth']

    def set_monthreshold(self, monthreshold):
        self.options['monthreshold'] = monthreshold

    def get_monthreshold(self):
        return self.options['monthreshold']

    def set_state(self, state):
        self.options['state'] = state

    def get_state(self):
        return self.options['state']

    def set_downstateflush(self, downstateflush):
        self.options['downstateflush'] = downstateflush

    def get_downstateflush(self):
        return self.options['downstateflush']

    def set_tcpprofilename(self, tcpprofilename):
        self.options['tcpprofilename'] = tcpprofilename

    def get_tcpprofilename(self):
        return self.options['tcpprofilename']

    def set_httpprofilename(self, httpprofilename):
        self.options['httpprofilename'] = httpprofilename

    def get_httpprofilename(self):
        return self.options['httpprofilename']

    def set_comment(self, comment):
        self.options['comment'] = comment

    def get_comment(self):
        return self.options['comment']

    def set_servername(self, servername):
        self.options['servername'] = servername

    def get_servername(self):
        return self.options['servername']

    def set_port(self, port):
        self.options['port'] = port

    def get_port(self):
        return self.options['port']

    def set_weight(self, weight):
        self.options['weight'] = weight

    def get_weight(self):
        return self.options['weight']

    def set_serverid(self, serverid):
        self.options['serverid'] = serverid

    def get_serverid(self):
        return self.options['serverid']

    def get_monitor_name_svc(self):
        return self.options['monitor_name_svc']

    def set_monitor_name_svc(self, monitor_name_svc):
        self.options['monitor_name_svc'] = monitor_name_svc

    def set_dup_weight(self, dup_weight):
        self.options['dup_weight'] = dup_weight

    def get_dup_weight(self):
        return self.options['dup_weight']

    def set_delay(self, delay):
        self.options['delay'] = delay

    def get_delay(self):
        return self.options['delay']

    def set_graceful(self, graceful):
        self.options['graceful'] = graceful

    def get_graceful(self):
        return self.options['graceful']

    def set_includemembers(self, includemembers):
        self.options['includemembers'] = includemembers

    def get_includemembers(self):
        return self.options['includemembers']

    def set_newname(self, newname):
        self.options['newname'] = newname

    def get_newname(self):
        return self.options['newname']

    def get_serviceconftype(self):
        return self.options['serviceconftype']

    def get_value(self):
        return self.options['value']

    def get_svrstate(self):
        return self.options['svrstate']

    def get_ip(self):
        return self.options['ip']

    def get_monitostate(self):
        return self.options['monitostate']

    def get_monstatcode(self):
        return self.options['monstatcode']

    def get_monstatparam1(self):
        return self.options['monstatparam1']

    def get_monstatparam2(self):
        return self.options['monstatparam2']

    def get_monstatparam3(self):
        return self.options['monstatparam3']

    def get_statechangetimesec(self):
        return self.options['statechangetimesec']

    def get_statechangetimemsec(self):
        return self.options['statechangetimemsec']

    def get_timesincelaststatechange(self):
        return self.options['timesincelaststatechange']

    def get_tickssincelaststatechange(self):
        return self.options['tickssincelaststatechange']

    def get_stateupdatereason(self):
        return self.options['stateupdatereason']

    def get_groupcount(self):
        return self.options['groupcount']

    # Operations methods
    @staticmethod
    def disable(nitro, servicegroup):
        """
        Use this API to disable servicegroup.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.set_delay(servicegroup.get_delay())
        return __servicegroup.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, servicegroup):
        """
        Use this API to enable servicegroup.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        return __servicegroup.perform_operation(nitro, "enable")

    @staticmethod
    def disable_server(nitro, servicegroup):
        """
        Use this API to disable server within servicegroup (Not globally).
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.set_servername(servicegroup.get_servername())
        __servicegroup.set_port(servicegroup.get_port())
        __servicegroup.set_delay(servicegroup.get_delay())
        return __servicegroup.perform_operation(nitro, "disable")

    @staticmethod
    def enable_server(nitro, servicegroup):
        """
        Use this API to enable server within servicegroup (Not globally).
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.set_servername(servicegroup.get_servername())
        __servicegroup.set_port(servicegroup.get_port())
        return __servicegroup.perform_operation(nitro, "enable")

    @staticmethod
    def rename(nitro, servicegroup):
        """
        Use this API to rename servicegroup.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.set_newname(servicegroup.get_newname())
        return __servicegroup.perform_operation(nitro, "rename")

    @staticmethod
    def get(nitro, servicegroup):
        """
        Use this API to fetch servicegroup resource of given name.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.get_resource(nitro, servicegroup.get_servicegroupname())
        return __servicegroup

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured servicegroup resources.
        """
        __url = nitro.get_url() + NSServiceGroup.get_resourcetype()
        __json_services = nitro.get(__url).get_response_field(NSServiceGroup.get_resourcetype())
        __servicegroups = []
        for json_service in __json_services:
            __servicegroups.append(NSServiceGroup(json_service))
        return __servicegroups

    @staticmethod
    def get_servers(nitro, servicegroup):
        """
        Use this API to fetch all configured servicegroup members.
        """
        __url = nitro.get_url() + 'servicegroup_servicegroupmember_binding' + '/' + servicegroup.get_servicegroupname()
        __json_services = nitro.get(__url).get_response_field('servicegroup_servicegroupmember_binding')
        __servicegroupmembers = []
        for json_service in __json_services:
            __servicegroupmembers.append(NSServiceGroup(json_service))
        return __servicegroupmembers

    @staticmethod
    def add(nitro, servicegroup):
        """
        Use this API to add service.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.set_servicetype(servicegroup.get_servicetype())
        __servicegroup.set_cachetype(servicegroup.get_cachetype())
        __servicegroup.set_maxclient(servicegroup.get_maxclient())
        __servicegroup.set_maxreq(servicegroup.get_maxreq())
        __servicegroup.set_cacheable(servicegroup.get_cacheable())
        __servicegroup.set_cip(servicegroup.get_cip())
        __servicegroup.set_cipheader(servicegroup.get_cipheader())
        __servicegroup.set_usip(servicegroup.get_usip())
        __servicegroup.set_useproxyport(servicegroup.get_useproxyport())
        __servicegroup.set_sc(servicegroup.get_sc())
        __servicegroup.set_sp(servicegroup.get_sp())
        __servicegroup.set_rtspsessionidremap(servicegroup.get_rtspsessionidremap())
        __servicegroup.set_clttimeout(servicegroup.get_clttimeout())
        __servicegroup.set_svrtimeout(servicegroup.get_svrtimeout())
        __servicegroup.set_cka(servicegroup.get_cka())
        __servicegroup.set_tcpb(servicegroup.get_tcpb())
        __servicegroup.set_cmp(servicegroup.get_cmp())
        __servicegroup.set_maxbandwidth(servicegroup.get_maxbandwidth())
        __servicegroup.set_monthreshold(servicegroup.get_monthreshold())
        __servicegroup.set_state(servicegroup.get_state())
        __servicegroup.set_downstateflush(servicegroup.get_downstateflush())
        __servicegroup.set_tcpprofilename(servicegroup.get_tcpprofilename())
        __servicegroup.set_httpprofilename(servicegroup.get_httpprofilename())
        __servicegroup.set_comment(servicegroup.get_comment())
        return __servicegroup.add_resource(nitro)

    @staticmethod
    def update(nitro, servicegroup):
        """
        Use this API to update servicegroup.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        __servicegroup.set_servername(servicegroup.get_servername())
        __servicegroup.set_port(servicegroup.get_port())
        __servicegroup.set_serverid(servicegroup.get_serverid())
        __servicegroup.set_monitor_name_svc(servicegroup.get_monitor_name_svc())
        __servicegroup.set_dup_weight(servicegroup.get_dup_weight())

        __servicegroup.set_maxclient(servicegroup.get_maxclient())
        __servicegroup.set_maxreq(servicegroup.get_maxreq())
        __servicegroup.set_cacheable(servicegroup.get_cacheable())
        __servicegroup.set_cip(servicegroup.get_cip())
        __servicegroup.set_cipheader(servicegroup.get_cipheader())
        __servicegroup.set_usip(servicegroup.get_usip())
        __servicegroup.set_useproxyport(servicegroup.get_useproxyport())
        __servicegroup.set_sc(servicegroup.get_sc())
        __servicegroup.set_sp(servicegroup.get_sp())
        __servicegroup.set_rtspsessionidremap(servicegroup.get_rtspsessionidremap())
        __servicegroup.set_clttimeout(servicegroup.get_clttimeout())
        __servicegroup.set_svrtimeout(servicegroup.get_svrtimeout())
        __servicegroup.set_cka(servicegroup.get_cka())
        __servicegroup.set_tcpb(servicegroup.get_tcpb())
        __servicegroup.set_cmp(servicegroup.get_cmp())
        __servicegroup.set_maxbandwidth(servicegroup.get_maxbandwidth())
        __servicegroup.set_monthreshold(servicegroup.get_monthreshold())
        __servicegroup.set_downstateflush(servicegroup.get_downstateflush())
        __servicegroup.set_tcpprofilename(servicegroup.get_tcpprofilename())
        __servicegroup.set_httpprofilename(servicegroup.get_httpprofilename())
        __servicegroup.set_comment(servicegroup.get_comment())
        return __servicegroup.update_resource(nitro)

    @staticmethod
    def delete(nitro, servicegroup):
        """
        Use this API to delete service of a given name.
        """
        __servicegroup = NSServiceGroup()
        __servicegroup.set_servicegroupname(servicegroup.get_servicegroupname())
        nsresponse = __servicegroup.delete_resource(nitro, servicegroup.get_servicegroupname())
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'vlazarenko'


class NSRewriteAction(NSBaseResource):

    # Configuration for content-switching rewrite action resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSRewriteAction, self).__init__()

        self.options = {
            'name': '',
            'type': '',
            'target': '',
            'stringbuilderexpr': '',
            'pattern': '',
            'search': '',
            'allow_unsafe_pi': '',
            'refinesearch': '',
            'comment': '',
            'allow_unsafe_pi1': '',
            'newname': '',
            'hits': '',
            'undefhits': '',
            'referencecount': '',
            'description': '',
        }

        self.resourcetype = NSRewriteAction.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "rewriteaction"

    def set_name(self, name):
        self.options['name'] = name

    def get_name(self):
        return self.options['name']

    def set_type(self, type):
        self.options['type'] = type

    def get_type(self):
        return self.options['type']

    def set_target(self, target):
        self.options['target'] = target

    def get_target(self):
        return self.options['target']

    def set_stringbuilderexpr(self, stringbuilderexpr):
        self.options['stringbuilderexpr'] = stringbuilderexpr

    def get_stringbuilderexpr(self):
        return self.options['stringbuilderexpr']

    def set_pattern(self, pattern):
        self.options['pattern'] = pattern

    def get_pattern(self):
        return self.options['pattern']

    def set_search(self, search):
        self.options['search'] = search

    def get_search(self):
        return self.options['search']

    def set_allow_unsafe_pi(self, allow_unsafe_pi):
        self.options['allow_unsafe_pi'] = allow_unsafe_pi

    def get_allow_unsafe_pi(self):
        return self.options['allow_unsafe_pi']

    def set_refinesearch(self, refinesearch):
        self.options['refinesearch'] = refinesearch

    def get_refinesearch(self):
        return self.options['refinesearch']

    def set_comment(self, comment):
        self.options['comment'] = comment

    def get_comment(self):
        return self.options['comment']

    def set_allow_unsafe_pi1(self, allow_unsafe_pi1):
        self.options['allow_unsafe_pi1'] = allow_unsafe_pi1

    def get_allow_unsafe_pi1(self):
        return self.options['allow_unsafe_pi1']

    def set_newname(self, newname):
        self.options['newname'] = newname

    def get_newname(self):
        return self.options['newname']

    def get_hits(self):
        return self.options['hits']

    def get_undefhits(self):
        return self.options['undefhits']

    def get_referencecount(self):
        return self.options['referencecount']

    def get_description(self):
        return self.options['description']

    # Operations methods
    @staticmethod
    def get(nitro, rewriteaction):
        """
        Use this API to fetch rewriteaction resource of given name.
        """
        __rewriteaction = NSRewriteAction()
        __rewriteaction.set_name(rewriteaction.get_name())
        __rewriteaction.get_resource(nitro)
        return __rewriteaction

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured rewriteaction resources.
        """
        __url = nitro.get_url() + NSRewriteAction.get_resourcetype()
        __json_cspolicies = nitro.get(__url).get_response_field(NSRewriteAction.get_resourcetype())
        __rewritepolicies = []
        for json_rewriteaction in __json_cspolicies:
            __rewritepolicies.append(NSRewriteAction(json_rewriteaction))
        return __rewritepolicies

    @staticmethod
    def add(nitro, rewriteaction):
        """
        Use this API to add rewriteaction.
        """
        __rewriteaction = NSRewriteAction()
        __rewriteaction.set_name(rewriteaction.get_name())
        __rewriteaction.set_type(rewriteaction.get_type())
        __rewriteaction.set_target(rewriteaction.get_target())
        __rewriteaction.set_stringbuilderexpr(rewriteaction.get_stringbuilderexpr())
        __rewriteaction.set_pattern(rewriteaction.get_pattern())
        __rewriteaction.set_search(rewriteaction.get_search())
        __rewriteaction.set_allow_unsafe_pi(rewriteaction.get_allow_unsafe_pi())
        __rewriteaction.set_refinesearch(rewriteaction.get_refinesearch())
        __rewriteaction.set_comment(rewriteaction.get_comment())

        return __rewriteaction.add_resource(nitro)

    @staticmethod
    def delete(nitro, rewriteaction):
        """
        Use this API to delete rewriteaction of a given name.
        """
        __rewriteaction = NSRewriteAction()
        __rewriteaction.set_name(rewriteaction.get_name())
        nsresponse = __rewriteaction.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, rewriteaction):
        """
        Use this API to update a rewriteaction of a given name.
        """
        __rewriteaction = NSRewriteAction()
        __rewriteaction.set_name(rewriteaction.get_name())
        __rewriteaction.set_target(rewriteaction.get_target())
        __rewriteaction.set_stringbuilderexpr(rewriteaction.get_stringbuilderexpr())
        __rewriteaction.set_pattern(rewriteaction.get_pattern())
        __rewriteaction.set_search(rewriteaction.get_search())
        __rewriteaction.set_allow_unsafe_pi(rewriteaction.get_allow_unsafe_pi())
        __rewriteaction.set_allow_unsafe_pi1(rewriteaction.get_allow_unsafe_pi1())
        __rewriteaction.set_refinesearch(rewriteaction.get_refinesearch())
        __rewriteaction.set_comment(rewriteaction.get_comment())
        return __rewriteaction.update_resource(nitro)

    # No unset functionality for now.
    @staticmethod
    def rename(nitro, rewriteaction):
        """
        Use this API to rename rewriteaction.
        """
        __rewriteaction = NSRewriteAction()
        __rewriteaction.set_name(rewriteaction.get_name())
        __rewriteaction.set_newname(rewriteaction.get_newname())
        return __rewriteaction.perform_operation(nitro, "rename")
